// ************************************************************************
//	Copyright (C) 1998. Bernoulli Project, Cornell University.
// ************************************************************************
#include "../src/config.h"
#include <assert.h>

#include "mex.h"
#include "matrix.h"

#include "brn_utils.h"
#include "brn_ccs.h"

// function x = BrnCcsForw2(D,LL,b)
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    mxAssert(nlhs <= 1, "Expected 1 lhs.");
    mxAssert(nrhs == 3, "Expected 3 lhs.");

    DenseVector D(prhs[0]);

    mxAssert(D.valid_p(), "D must be a real dense vector.");

    CCS LL(prhs[1]);
    mxAssert(LL.valid_p(), "LL must be a real sparse array.");
    mxAssert(LL.m() == LL.n(), "Currently, LL must square.");

    mxAssert(D.n() == LL.n(), "Ranks of D and LL are not compatible.");

    DenseVector b(prhs[2]);
    mxAssert(b.valid_p(), "b must be a real dense vector.");

    mxAssert(D.n() == b.n(), "Ranks of LL and b are not compatible.");

    plhs[0] = mxCreateDoubleMatrix(D.n(),1,mxREAL);
    DenseVector x(plhs[0]);

    ccs_forward_right(
	D.n(), 
	LL.nzs(), LL.colptr(), LL.rowind(), LL.value(), 
	D.value(), x.value(), b.value());
}
