// ************************************************************************
//	Copyright (C) 1998. Bernoulli Project, Cornell University.
// ************************************************************************
#include "../src/config.h"
#include <assert.h>

#include "mex.h"
#include "matrix.h"

#include "brn_utils.h"
#include "brn_crs.h"

// function y = BrnCrsMvm(A,x)
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    mxAssert(nlhs <= 1, "Expected 1 lhs.");
    mxAssert(nrhs == 2, "Expected 2 lhs.");

    CRS A(prhs[0]);
    mxAssert(A.valid_p(), "A must be a real sparse array.");
    mxAssert(A.m() == A.n(),"Currently, A must square.");

    DenseVector x(prhs[1]);
    mxAssert(x.valid_p(),"x must be a real dense vector.");

    mxAssert(A.m() == x.n(), "Ranks of A and x are not compatible.");

    plhs[0] = mxCreateDoubleMatrix(A.n(),1,mxREAL);
    DenseVector y(plhs[0]);

    crs_mvm(A.n(), A.nzs(), A.rowptr(), A.colind(), A.value(),
	    y.value(), x.value());

}
