// ************************************************************************
//	Copyright (C) 1998. Bernoulli Project, Cornell University.
// ************************************************************************
#include "../src/config.h"
#include <assert.h>

#include "mex.h"
#include "matrix.h"

#pragma implementation
#include "brn_utils.h"

int destruct_dense_matrix(
    const mxArray *a, 
    int *m, int *n, double **value)
{
    if (!(mxGetClassID(a) == mxDOUBLE_CLASS && !mxIsComplex(a)))
	return 0;

    *m = mxGetM(a);
    *n = mxGetN(a);
    *value = mxGetPr(a);
    return 1;
}

int destruct_dense_vector(
    const mxArray *a, 
    int *n, double **value)
{
    if (!(mxGetClassID(a) == mxDOUBLE_CLASS && !mxIsComplex(a)))
	return 0;

    // Assumes that n-length vectors a stored as n-1 matrices.
    if (!(mxGetN(a) == 1))
	return 0;
    *n = mxGetM(a);
    *value = mxGetPr(a);
    return 1;
}

void crs_to_ccs(int m, int n, int nzs,
		   int *in_rowptr, int *in_colind, double *in_value,
		   int *out_colptr, int *out_rowind, double *out_value)
{
    // Count the number of nzs in each column. 
    // Store results in out_colptr.
    int *out_nzs_per_col = (int *)mxCalloc(n, sizeof(int));
    {for (int i=0; i<m; i++)
	for (int jj=in_rowptr[i]; jj<in_rowptr[i+1]; jj++)
	    out_nzs_per_col[in_colind[jj]]++;}

    // Set the col ptrs
    {
	int ii=0;
	for (int j=0; j<n; j++) {
	    out_colptr[j] = ii;
	    ii += out_nzs_per_col[j];
	}

	assert(ii == nzs);
	out_colptr[n] = nzs;
    }

    // Copy the nzs in each column.
    {for (int j=0; j<n; j++) out_nzs_per_col[j] = 0;}
    {for (int i=0; i<m; i++)
	for (int jj=in_rowptr[i]; jj<in_rowptr[i+1]; jj++) {
	    int j = in_colind[jj];
	    double v = in_value[jj];
	    int ii = out_colptr[j] + out_nzs_per_col[j];
	    assert(ii < out_colptr[j+1]);
	    out_nzs_per_col[j]++;
	    out_rowind[ii] = i;
	    out_value[ii] = v;
	}}

    mxFree(out_nzs_per_col);
}

void ccs_to_crs(int m, int n, int nzs,
		int *in_colptr, int *in_rowind, double *in_value,
		int *out_rowptr, int *out_colind, double *out_value)
{
    crs_to_ccs(n,m,nzs,
	       in_colptr, in_rowind, in_value,
	       out_rowptr, out_colind, out_value);
}
