/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
/* Vladimir Kotlyar August 1996 */

#include "./config.h"

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "alpha_io2.h"
#include "alpha.h"

int xa_cons_count = 0;

xa_calc_var	
a_make_calc_var (arena_info *arena, char *name)
{
  xa_calc_var v;
  int n = strlen(name);
  assert (name);
  v = a_alloc0 (arena, xa_calc_var_t);
  xa_name_calc_var (v) = a_nalloc0(arena,char,n+1);
  strcpy (xa_name_calc_var(v), name);
  return v;
}

xa_calc_var
a_make_temp_calc_var (arena_info *arena, int i)
{
  static char buf[100];
  assert (i >= 0);
  sprintf (buf, "t%d", i);
  return a_make_calc_var (arena, buf);
}

xa_calc_var
a_make_copy_calc_var (arena_info *arena, xa_calc_var v)
{
  return a_make_calc_var (arena, xa_name_calc_var(v));
}

xa_calc_symtab  
a_make_calc_symtab (arena_info *arena, 
		    A_CONS_CELL(xa_calc_var) set_vars,
		    A_CONS_CELL(xa_calc_var) exist_vars)
{
  int	n;
  int	nprot;
  int	i;
  xa_calc_var *table = 0;
  xa_calc_symtab t;

  n = 0;
  nprot = 0;
  A_FOR_CONS_CHAIN(xa_calc_var,set_vars,p) {
    n++;
    nprot++;
  }
  A_END_CONS_CHAIN;

  A_FOR_CONS_CHAIN(xa_calc_var,exist_vars,p) {
    n++;
  }
  A_END_CONS_CHAIN;

  if (n) {
    table = a_nalloc0 (arena, xa_calc_var, n);
    
    i = 0;
    
    A_FOR_CONS_CHAIN(xa_calc_var,set_vars,p) {
      table[i] = a_car(p);
      i++;
    }
    A_END_CONS_CHAIN;
    
    A_FOR_CONS_CHAIN(xa_calc_var,exist_vars,p) {
      table[i] = a_car(p);
      i++;
    }
    A_END_CONS_CHAIN;
  }
    
  t = a_alloc0 (arena, xa_calc_symtab_t);
  xa_n_calc_symtab(t) = n;
  xa_nprot_calc_symtab(t) = nprot;
  xa_table_calc_symtab(t) = table;

  return t;
}

int
a_pos4var_calc_symtab (xa_calc_symtab t, xa_calc_var v)
{
  int	i;
  for (i = 1; i <= xa_n_calc_symtab(t); i++) {
    if (0 == 
	strcmp(xa_name_calc_var(v),
	       xa_name_calc_var(xa_var_calc_symtab(t,i)))) {
      return i;
    }
  }
  return 0;
}

int
a_is_protected_var_calc_symtab (xa_calc_symtab t, xa_calc_var v)
{
  int i = a_pos4var_calc_symtab(t,v);

  assert(i);

  return (i <= xa_nprot_calc_symtab(t));
}

void
xa_verify_calc_symtab (xa_calc_symtab t)
{
  int i;
#if !NDEBUG
  /* make sure there are no duplicates */
  for (i = 1; i <= xa_n_calc_symtab(t); i++) {
    xa_calc_var v = xa_var_calc_symtab(t,i);
    assert (i == a_pos4var_calc_symtab(t, v));
  }
#endif
}

void
xa_dump_calc_symtab (FILE *fp, xa_calc_symtab t)
{
  int i;
  fprintf (fp, "symtab %d %d: ", 
	   xa_n_calc_symtab(t),
	   xa_nprot_calc_symtab(t));

  for (i = 1; i <= xa_n_calc_symtab(t); i++) {
    a_print_calc_var (fp, xa_var_calc_symtab(t, i));
  }
}


xa_calc_term
a_make_calc_term (arena_info *arena, xa_calc_var var, int coeff)
{
  xa_calc_term t = a_alloc0 (arena, xa_calc_term_t);
  xa_var_calc_term (t) = var;
  xa_coeff_calc_term (t) = coeff;

  return t;
}

xa_calc_constraint
a_make_calc_constraint (arena_info *arena, xa_calc_op op,
			A_CONS_CELL(xa_calc_term) t1,
			A_CONS_CELL(xa_calc_term) t2)
{
  xa_calc_constraint  c = a_alloc0(arena, xa_calc_constraint_t);
  xa_calc_term  trm;
  assert (op);
  if (!t1) {
    trm = a_make_calc_term (arena, 0, 0);
    a_make_cons (arena, xa_calc_term, t1, trm, t1);
  }
  if (!t2) {
    trm = a_make_calc_term (arena, 0, 0);
    a_make_cons (arena, xa_calc_term, t2, trm, t2);
  }
  xa_op_calc_constraint(c) = op;
  xa_t1_calc_constraint(c) = t1;
  xa_t2_calc_constraint(c) = t2;
  return c;
}

xa_calc_set
a_make_calc_set(arena_info *arena, 
		A_CONS_CELL(xa_calc_var) set_vars,
		A_CONS_CELL(xa_calc_var) exist_vars,
		A_CONS_CELL(xa_calc_constraint) pred)
{
  xa_calc_set s = a_alloc0(arena, xa_calc_set_t);
  xa_set_vars_calc_set(s) = set_vars;
  xa_exist_vars_calc_set(s) = exist_vars;
  xa_pred_calc_set(s) = pred;

  return s;
}

void
a_print_calc_var (FILE *fp, xa_calc_var v)
{
  fprintf (fp, "%s", xa_name_calc_var (v));
}

void
a_print_calc_term (FILE *fp, xa_calc_term t, int mult)
{
  int c = mult * xa_coeff_calc_term (t);

  if (xa_is_const_calc_term(t)) {
    fprintf (fp, "%d", c);
  }
  else {
    if (c == 1) {
      fprintf (fp, "%s", xa_name_calc_var(xa_var_calc_term(t)));
    }
    else if (c == -1) {
      fprintf (fp, "-%s", xa_name_calc_var(xa_var_calc_term(t)));
    }
    else {
      fprintf (fp, "%d*%s", c, xa_name_calc_var(xa_var_calc_term(t)));
    }
  }
}

static void
xa_print_term_cons (FILE *fp, A_CONS_CELL(xa_calc_term) p)
{
  A_CONS_CELL(xa_calc_term) next;
  int mult=1;

  assert (p);

  A_FOR_CONS_CHAIN(xa_calc_term,p,q) {
    a_print_calc_term (fp, a_car(q), mult);
    next = a_cdr(q);
    if (next) {
      if (0 > xa_coeff_calc_term(a_car(next))) {
	fprintf (fp, "-");
	mult = -1;
      }
      else if (0 < xa_coeff_calc_term(a_car(next))){
	fprintf (fp, "+");
	mult = 1;
      }
    }
  }
  A_END_CONS_CHAIN;
}

void
a_print_calc_constraint (FILE *fp, xa_calc_constraint c)
{
  static char * op_names [XA_CALC_LAST_OP] = 
  { "error", "=", "<=", "<", ">=", ">" };

  xa_print_term_cons (fp, xa_t1_calc_constraint(c));
  fprintf (fp, " %s ", op_names[xa_op_calc_constraint(c)]);
  xa_print_term_cons (fp, xa_t2_calc_constraint(c));
}

void
a_print_calc_set (FILE *fp, xa_calc_set s)
{
  if (0 == s) {
    fprintf (fp, "EMPTY");
  }

  fprintf (fp, "{");
  A_FOR_CONS_CHAIN(xa_calc_var,xa_set_vars_calc_set(s),p) {
    a_print_calc_var (fp, a_car(p));
    if (a_cdr(p)) {
      fprintf (fp, ",");
    }
  }
  A_END_CONS_CHAIN;
  fprintf (fp, "| exist ");
  A_FOR_CONS_CHAIN(xa_calc_var,xa_exist_vars_calc_set(s),p) {
    a_print_calc_var (fp, a_car(p));
    if (a_cdr(p)) {
      fprintf (fp, ",");
    }
  }
  A_END_CONS_CHAIN;
  fprintf (fp, " : ");
  A_FOR_CONS_CHAIN(xa_calc_constraint,xa_pred_calc_set(s),p) {
    a_print_calc_constraint (fp, a_car(p));
    if (a_cdr(p)) {
      fprintf (fp, " & ");
    }    
  }
  A_END_CONS_CHAIN;
  fprintf (fp, "};");
}

/**********************************************************************/

static void
constraint2matrix (xa_calc_constraint c, 
		   int add, int mult1, int mult2,
		   int i, int n, int m,
		   AlphaMatrix A, AlphaVector B,
		   xa_calc_symtab t)
{
  int pos;
  xa_calc_var v;
  int coeff;

  assert (n == a_cols_mx (A));
  assert (m == a_rows_mx (A));
  assert (m == a_length_vec (B));
  assert (1 <= i && i <= m);

  A_FOR_CONS_CHAIN(xa_calc_term, xa_t1_calc_constraint(c), p) {
    v = xa_var_calc_term(a_car(p));
    coeff = xa_coeff_calc_term(a_car(p));
    if (v) {
      pos = a_pos4var_calc_symtab (t, v);
      assert (pos && (pos <= n));
      a_elt_mx (A, i, pos) += mult1 * coeff;
    }
    else {
      a_elt_vec (B, i) -= mult1 * coeff; /* constants are on the other side */
    }
  }
  A_END_CONS_CHAIN;

  A_FOR_CONS_CHAIN(xa_calc_term, xa_t2_calc_constraint(c), p) {
    v = xa_var_calc_term(a_car(p));
    coeff = xa_coeff_calc_term(a_car(p));
    if (v) {
      pos = a_pos4var_calc_symtab (t, v);
      assert (pos && (pos <= n));
      a_elt_mx (A, i, pos) += mult2 * coeff;
    }
    else {
      a_elt_vec (B, i) -= mult2 * coeff;
    }
  }
  A_END_CONS_CHAIN;

  a_elt_vec (B,i) += add;
}

/************************************************************************
 *		      A set -> A * X == B, F * X <= G                   *
 ************************************************************************/


static void set2matrices(arena_info *arena, xa_calc_set s, 
			 int *zn, int *znprot, xa_calc_symtab *zt,
			 AlphaMatrix *zA, AlphaVector *zB,
			 AlphaMatrix *zF, AlphaVector *zG)
{
  AlphaMatrix A;
  AlphaVector B;
  AlphaMatrix F;
  AlphaVector G;
  int n, nprot, meq, mineq;
  int mult1, mult2;
  int add1, add2;
  int ieq, iineq;
  xa_calc_symtab t;

  double t_to_matrix, t_project, t_to_set;

  t = a_make_calc_symtab (arena, xa_set_vars_calc_set(s), 
			  xa_exist_vars_calc_set(s));
  n = xa_n_calc_symtab(t);
  nprot = xa_nprot_calc_symtab(t);

  meq = 0;
  mineq = 0;
  A_FOR_CONS_CHAIN(xa_calc_constraint,xa_pred_calc_set(s),p) {
    if (xa_op_calc_constraint(a_car(p)) == XA_CALC_EQ) {
      meq ++;
    }
    else {
      mineq ++;
    }
  }
  A_END_CONS_CHAIN;

  A = a_make_matrix (arena, meq, n);
  B = a_make_vector (arena, meq);
  F = a_make_matrix (arena, mineq, n);
  G = a_make_vector (arena, mineq);

  ieq = 1;
  iineq = 1;
  A_FOR_CONS_CHAIN(xa_calc_constraint,xa_pred_calc_set(s),p) {
    switch (xa_op_calc_constraint(a_car(p))) {
    case XA_CALC_EQ:
      constraint2matrix (a_car(p), 0, 1, -1, 
			 ieq, n, meq, 
			 A, B, t);
      ieq ++;
      break;
    case XA_CALC_LE:
      constraint2matrix (a_car(p), 0, 1, -1, 
			 iineq, n, mineq, 
			 F, G, t);
      iineq ++;
      break;
    case XA_CALC_LT:
      constraint2matrix (a_car(p), -1, 1, -1, 
			 iineq, n, mineq, 
			 F, G, t);
      iineq ++;
      break;
    case XA_CALC_GE:
      constraint2matrix (a_car(p), 0, -1, 1, 
			 iineq, n, mineq, 
			 F, G, t);
      iineq ++;
      break;
    case XA_CALC_GT:
      constraint2matrix (a_car(p), -1, -1, 1, 
			 iineq, n, mineq, 
			 F, G, t);
      iineq ++;
      break;
    default:
      assert (0);
    }
  }
  A_END_CONS_CHAIN;

  *zn = n;
  *znprot = nprot;
  *zt = t;
  *zA = A;
  *zB = B;
  *zF = F;
  *zG = G;
}

/************************************************************************
 *			   A projection -> a set                        *
 ************************************************************************/

static xa_calc_set
proj2set (arena_info *arena, xa_proj P, xa_calc_symtab t)
{
  AlphaIneqList		C;
  int			m_ineq;
  AlphaMatrix		H;
  AlphaVector		X0;
  int			m;
  int			n;
  int			nvars = a_nvars_proj (P);
  int			nprot = a_nprot_proj (P);
  int			n_unprot = nvars - nprot;
  AlphaMatrix 	        Sigma, Sigma_non_unit, U, V;
  AlphaMatrix		U1, U2, U3;
  AlphaMatrix		F;
  AlphaVector		G;
  AlphaMatrix		Vblock, Ublock, Iunprot, Inonunit, F2, F3;
  AlphaVector		G2, U1_X0, U2_X0, U3_X0, F2_U1_X0;
  int			r, r_unit;
  int			i, j, k;

  A_CONS_CELL(xa_calc_var) s_vars=0, e_vars=0;
  A_CONS_CELL(xa_calc_constraint) pred=0;
  xa_calc_symtab new_t;
  xa_calc_set new_set;

  C = a_ineq_list_proj (P);
  m_ineq = a_length_ineq_list (C);
  X0 = a_X0_proj (P);
  H = a_H_proj (P);
  m = a_rows_mx (H);
  n = a_cols_mx (H);

  assert (n == nprot);
  assert (m == a_length_vec (X0));

  Sigma = a_make_matrix (arena, m, n);
  U = a_make_matrix (arena, m, m);
  V = a_make_matrix (arena, n, n);

  a_copy_mx (Sigma, H);
  a_snf (arena, Sigma, U, V, &r);

    
  for (i = 1; i <= r && (a_elt_mx(Sigma,i,i) == 1); i++) {
    /* just count */
  }
  r_unit = i - 1;

  Sigma_non_unit = a_make_matrix (arena, r-r_unit, r-r_unit);
  if (r - r_unit > 0) {
    a_extract (Sigma, r_unit+1, r, r_unit+1, r, Sigma_non_unit);
  }

  U1 = a_make_matrix (arena, r_unit, m);
  U2 = a_make_matrix (arena, r - r_unit, m);
  U3 = a_make_matrix (arena, m - r, m);

  if (r_unit > 0) {
    a_extract (U, 1, r_unit, 1, m, U1);
  }

  if (r - r_unit > 0) {
    a_extract (U, r_unit+1, r, 1, m, U2);
  }

  if (m - r > 0) {
    a_extract (U, r+1, m, 1, m, U3);
  }
    
  F = a_make_matrix (arena, m_ineq, nvars);
  G = a_make_vector (arena, m_ineq);

  a_ineq_list2matrix_form (C, F, G);

  assert (nvars == n + n_unprot);

  Vblock = a_make_matrix (arena, nvars, nvars);
  Iunprot = a_make_matrix (arena, n_unprot, n_unprot);
  a_set2ident (Iunprot);
  if (n > 0) {
    a_replace (Vblock, 1, n, 1, n, V);
  }
  if (nvars >= n + 1) {
    a_replace (Vblock, n+1, nvars, n+1, nvars, Iunprot);
  }

  Inonunit = a_make_matrix (arena, nvars - r_unit, nvars - r_unit);
  a_set2ident (Inonunit);
  Ublock = a_make_matrix (arena, nvars, m + nvars - r_unit);
  if (r_unit > 0) {
    a_replace (Ublock, 1, r_unit, 1, m, U1);
  }
  if ((nvars >= r_unit + 1) && (m + nvars - r_unit >= m + 1)) {
    a_replace (Ublock, r_unit+1, nvars, m+1, m + nvars - r_unit, Inonunit);
  }

  F2 = a_make_matrix (arena, m_ineq, nvars);
  F3 = a_make_matrix (arena, m_ineq, m + nvars - r_unit);

  a_matmult (arena, F, Vblock, F2);
  a_matmult (arena, F2, Ublock, F3);

  G2 = a_make_vector (arena, m_ineq);
  U1_X0 = a_make_vector (arena, r_unit);
  a_mvm (arena, U1, X0, U1_X0);
  a_neg_vec (U1_X0);
    
  a_resize_vec (arena, & U1_X0, nvars);

  F2_U1_X0 = a_make_vector (arena, m_ineq);

  a_mvm (arena, F2, U1_X0, F2_U1_X0);

  a_saxpy (1, G, -1, F2_U1_X0, G2);

  /* F3 and G2 give us the new inequalities */

  /* 
  a_fprintf (a_log_file, 
	     "The variables are:\n"
	     "\toriginal %d non-unit %d unprotected left %d\n",
	     m, nvars - r_unit, n_unprot);

  a_fprintf (a_log_file,
	     "Sigma_nu=%M U2=%M\nU3 = %M\nF3=%M\nG2=%V\n",
	     Sigma_non_unit, U2, U3, F3, G2);
  */

  /* At this point we have:
     - two sets of variables x and z
     - |x| = m
     - |z| = nvars - r_unit
     - x are the old set (protected) variables
     - z are existentially quantified
     - first (r - r_unit) variables of z (call them z1) are related to x by
     stride constraints:
     U2*(x-x0) == Sigma_non_unit*z1
     - there are equality constraints purely on x:
     U3*(x-x0) == 0
     - and there are inequalitied in terms of x and z:
     F3*[x z] <= G2
     */

  /* lets verify the sizes */
  assert (a_rows_mx(U2) == r - r_unit);
  assert (a_rows_mx(Sigma_non_unit) == r - r_unit);
  assert (a_rows_mx(Sigma_non_unit) == r - r_unit);
  assert (a_rows_mx(U3) == m - r);
  assert (a_cols_mx(F3) == m + nvars - r_unit);
  assert (a_rows_mx(F3) == a_length_vec(G2));
  assert (a_rows_mx(F3) == m_ineq);

  assert (m == xa_nprot_calc_symtab(t));
    
  for (i = m; i >= 1; i--) {
    xa_calc_var v = a_make_copy_calc_var (arena, xa_var_calc_symtab(t,i));
    a_make_cons (arena, xa_calc_var, s_vars, v, s_vars);
  }
    
  for (i = 1; i <= nvars - r_unit; i++) {
    xa_calc_var v = a_make_temp_calc_var (arena, i);
    a_make_cons (arena, xa_calc_var, e_vars, v, e_vars);
  }

  new_t = a_make_calc_symtab (arena, s_vars, e_vars);

  /* now we are ready to fill in the terms */

  /*   equalities */
    
  /*      stride constraints  U2*x-U2*x0 == Sigma_non_unit*z1 */
  U2_X0 = a_make_vector (arena, r - r_unit);
  a_mvm (arena, U2, X0, U2_X0);

  for (i = 1; i <= r - r_unit; i++) {
    A_CONS_CELL(xa_calc_term) t1, t2;
    xa_calc_constraint c;
      
    xa_calc_term trm;
    xa_calc_var v;

    t1 = 0;
    t2 = 0;
    for (j = 1; j <= m; j++) {
      v = xa_var_calc_symtab (new_t, j);
      if (a_elt_mx (U2, i, j) != 0) {
	trm = a_make_calc_term (arena, v, a_elt_mx (U2, i, j));
	a_make_cons (arena, xa_calc_term, t1, trm, t1);
      }
    }

    if (0 != a_elt_vec (U2_X0,i)) {
      trm = a_make_calc_term (arena, 0, -a_elt_vec(U2_X0, i));
      a_make_cons (arena, xa_calc_term, t1, trm, t1);
    }

    v = xa_var_calc_symtab(new_t, m + i);
    trm = a_make_calc_term (arena, v, a_elt_mx(Sigma_non_unit, i, i));
    a_make_cons (arena, xa_calc_term, t2, trm, t2);
      
    assert (t1 || t2); /* must not have trivial constraints */

    c = a_make_calc_constraint(arena, XA_CALC_EQ, t1, t2);
    a_make_cons (arena, xa_calc_constraint, pred, c, pred);
  }

  /*      equality constraints  U3*(x - x0) == 0 */

  U3_X0 = a_make_vector (arena, m - r);
  a_mvm (arena, U3, X0, U3_X0);

  for (i = 1; i <= m - r; i++) {
    A_CONS_CELL(xa_calc_term) t1, t2;
    xa_calc_constraint c;      
    xa_calc_term trm;
    xa_calc_var v;

    t1 = 0;
    t2 = 0;

    for (j = 1; j <= m; j++) {
      v = xa_var_calc_symtab (new_t, j);
      if (0 != a_elt_mx (U3, i, j)) {
	trm = a_make_calc_term (arena, v, a_elt_mx (U3, i, j));
	a_make_cons (arena, xa_calc_term, t1, trm, t1);
      }
    }

    if (0 != a_elt_vec(U3_X0,i)) {
      trm = a_make_calc_term (arena, 0, a_elt_vec(U3_X0,i));
      a_make_cons (arena, xa_calc_term, t2, trm, t2);
    }

    assert (t1 || t2); /* must not have trivial constraints */

    c = a_make_calc_constraint(arena, XA_CALC_EQ, t1, t2);
    a_make_cons (arena, xa_calc_constraint, pred, c, pred);
  }

  /*  inequalities */

  for (i = 1; i <= m_ineq; i++) {
    A_CONS_CELL(xa_calc_term) t1, t2;
    int coeff;
    xa_calc_constraint c;      
    xa_calc_term trm;
    xa_calc_var v;

    t1 = 0;
    t2 = 0;

    for (j = 1; j <= m + nvars - r_unit; j++) {
      v = xa_var_calc_symtab (new_t, j);
      coeff = a_elt_mx (F3, i, j);
      if (coeff > 0) {
	trm = a_make_calc_term (arena, v, coeff);
	a_make_cons (arena, xa_calc_term, t1, trm, t1);		  
      }
      else if (coeff < 0) {
	trm = a_make_calc_term (arena, v, -coeff);
	a_make_cons (arena, xa_calc_term, t2, trm, t2);
      }
    }

    if (0 != a_elt_vec(G2,i)) {
      trm = a_make_calc_term (arena, 0, a_elt_vec(G2,i));
      a_make_cons (arena, xa_calc_term, t2, trm, t2);
    }

    assert (t1 || t2); /* must not have trivial constraints */

    c = a_make_calc_constraint(arena, XA_CALC_LE, t1, t2);
    a_make_cons (arena, xa_calc_constraint, pred, c, pred);
  }

  /* done ! */
  new_set = a_make_calc_set (arena, s_vars, e_vars, pred);

  return new_set;
}

/************************************************************************
 *			      Satisfiability                            *
 ************************************************************************/

int a_satisfiable_p(arena_info *arena, xa_calc_set s)
{
  xa_calc_symtab t;
  AlphaMatrix A;
  AlphaVector B;
  AlphaMatrix F;
  AlphaVector G;
  int n, nprot;

  /* do not splinter on symbolic projections */
  xa_splinter_symbolic = 0;

  /* Convert set to the matrices */
  set2matrices(arena, s, &n, &nprot, &t, &A, &B, &F, &G);

  assert(a_rows_mx(F) > 0);
  if (a_rows_mx(A) == 0)
      return a_decide_ineq(arena, F, G);
  else
      return a_decide_eq_and_ineq(arena, A, B, F, G);
}

/************************************************************************
 *			      Simplification                            *
 ************************************************************************/

xa_calc_set	
a_simplify_set_with_times (
    arena_info *arena, xa_calc_set s,
    double *zt_to_matrix, double *zt_project, double *zt_to_set)
{
  xa_proj_list acc;
  xa_proj P;
  xa_calc_symtab t;
  AlphaMatrix A;
  AlphaVector B;
  AlphaMatrix F;
  AlphaVector G;
  int n, nprot;
  xa_calc_set s_new;

  double t_to_matrix, t_project, t_to_set;

  /* do not splinter on symbolic projections */
  xa_splinter_symbolic = 0;

  /* Convert set to the matrices */
  reset_elapse_timer ();
  set2matrices(arena, s, &n, &nprot, &t, &A, &B, &F, &G);
  t_to_matrix = get_elapse_timer ();

  acc = a_symbolic_projection (arena, nprot, A, B, F, G);

  t_project = get_elapse_timer ();

  if (0 == acc) {
    return 0;
  }
	      
  P = a_car_proj_list (acc);

  /* assume no splintering for now */
  assert (0 == a_cdr_proj_list (acc));

  s_new = proj2set (arena, P, t);

  t_to_set = get_elapse_timer ();

  *zt_to_matrix = t_to_matrix;
  *zt_project = t_project;
  *zt_to_set = t_to_set;

  return s_new;
}

xa_calc_set	
a_simplify_set (arena_info *arena, xa_calc_set s)
{
  double t_to_matrix, t_project, t_to_set;
  return a_simplify_set_with_times(
      arena, s, &t_to_matrix, &t_project, &t_to_set);
}
