/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
/* Vladimir Kotlyar August 1996 */

/***********************************************************************
 *
 * Data structures for Alpha calculator interface
 *
 ***********************************************************************/

#ifndef _a_calc_h
#define _a_calc_h

#include <stdio.h>

#ifndef _a_memory_h
#include "alpha_memory.h"
#endif

#ifndef _a_range_h
#include "alpha_range.h"
#endif

/***************************** cons cells **********************/

extern int xa_cons_count;

#define A_CONS_CELL(type)   type##cons
#define XA_CONS_CELL_STRUCT(type) type##cons_s
#define XA_CONS_CELL_TYPE(type) type##cons_t

#define A_DEFINE_CONS_CELL(type)					\
typedef struct XA_CONS_CELL_STRUCT(type) * A_CONS_CELL(type);		\
typedef struct XA_CONS_CELL_STRUCT(type) XA_CONS_CELL_TYPE(type);	\
struct XA_CONS_CELL_STRUCT(type) {					\
  type car;								\
  A_CONS_CELL(type) cdr;						\
}
  
#define a_car(p)  ((p)->car)
#define a_cdr(p)  ((p)->cdr)

/* p = cons(q,c) */
#define a_make_cons(arena,type,p,q,c)		\
{ A_CONS_CELL(type) __make_cons_tmp;		\
  __make_cons_tmp = (c);			\
  (p) = a_alloc0(arena,XA_CONS_CELL_TYPE(type));	\
  a_car(p) = (q);				\
  a_cdr(p) = __make_cons_tmp;			\
  xa_cons_count++; }

#define A_FOR_CONS_CHAIN(type,start,p)		\
{						\
  A_CONS_CELL(type) __for_cons_chain_next;	\
  A_CONS_CELL(type) p;				\
  for (p = (start); p;				\
       p  = __for_cons_chain_next) {		\
    __for_cons_chain_next = a_cdr(p);

#define A_END_CONS_CHAIN }}

/* can be destructive */
#define A_MAP_CONS_CHAIN(type,start,func)	\
A_FOR_CONS_CHAIN(type,(start),p) {		\
  (func)(p);					\
}						\
A_END_CONS_CHAIN;


/********** variables ************/

typedef struct xa_calc_var_s xa_calc_var_t;
typedef struct xa_calc_var_s *xa_calc_var;

struct xa_calc_var_s {
  char *name;
};

#define xa_name_calc_var(v)	((v)->name)

A_DEFINE_CONS_CELL(xa_calc_var);

/********** simple (small) symbol table ************/

typedef struct xa_calc_symtab_s xa_calc_symtab_t;
typedef struct xa_calc_symtab_s *xa_calc_symtab;

struct xa_calc_symtab_s {
  int n;
  int nprot;
  xa_calc_var *table;
};

#define xa_size_calc_symtab(t)	((t)->n)
#define xa_n_calc_symtab(t)	((t)->n)
#define xa_nprot_calc_symtab(t)	((t)->nprot)
#define xa_var_calc_symtab(t,i)	((t)->table[xa_check_range((i),(t)->n),((i)-1)])
#define xa_table_calc_symtab(t)	((t)->table)


/********* terms *********/

typedef struct xa_calc_term_s xa_calc_term_t;
typedef struct xa_calc_term_s *xa_calc_term;

struct xa_calc_term_s {
  xa_calc_var  var;
  int coeff;
};

#define xa_var_calc_term(t)	((t)->var)
#define xa_coeff_calc_term(t)	((t)->coeff)
#define xa_is_const_calc_term(t) (0 == xa_var_calc_term(t))

A_DEFINE_CONS_CELL(xa_calc_term);

/********** constraints ***********/

typedef enum  {
  XA_CALC_UNKNOWN = 0,
  XA_CALC_EQ,
  XA_CALC_LE,
  XA_CALC_LT,
  XA_CALC_GE,
  XA_CALC_GT,
  XA_CALC_LAST_OP
} xa_calc_op;

typedef struct xa_calc_constraint_s xa_calc_constraint_t;
typedef struct xa_calc_constraint_s *xa_calc_constraint;

struct xa_calc_constraint_s {
  xa_calc_op  op;
  A_CONS_CELL(xa_calc_term) t1, t2;
};

#define xa_op_calc_constraint(c)	((c)->op)
#define xa_t1_calc_constraint(c)	((c)->t1)
#define xa_t2_calc_constraint(c)	((c)->t2)

A_DEFINE_CONS_CELL(xa_calc_constraint);

/************* set *****************/

typedef struct xa_calc_set_s xa_calc_set_t;
typedef struct xa_calc_set_s *xa_calc_set;

struct xa_calc_set_s {
  A_CONS_CELL(xa_calc_var)  		set_vars;
  A_CONS_CELL(xa_calc_var)  		exist_vars;
  A_CONS_CELL(xa_calc_constraint) 	pred;
};

#define xa_set_vars_calc_set(s)		((s)->set_vars)
#define xa_exist_vars_calc_set(s)	((s)->exist_vars)
#define xa_pred_calc_set(s)		((s)->pred)

/**********************************************************************
 *  Function prototypes
 **********************************************************************/

xa_calc_var	a_make_calc_var (arena_info *arena, char *name);
xa_calc_var	a_make_temp_calc_var (arena_info *arena, int i);
xa_calc_var	a_make_copy_calc_var (arena_info *arena, xa_calc_var v);

xa_calc_symtab  a_make_calc_symtab (arena_info *arena, 
				    A_CONS_CELL(xa_calc_var) set_vars,
				    A_CONS_CELL(xa_calc_var) exist_vars);
int		a_pos4var_calc_symtab (xa_calc_symtab t, xa_calc_var v);
int		a_is_protected_var_calc_symtab (xa_calc_symtab t, xa_calc_var v);
void		xa_verify_calc_symtab(xa_calc_symtab t);
void		xa_dump_calc_symtab (FILE *fp, xa_calc_symtab t);

xa_calc_term	a_make_calc_term (arena_info *arena, 
				  xa_calc_var var, int coeff);

xa_calc_constraint a_make_calc_constraint (arena_info *arena, xa_calc_op op,
					   A_CONS_CELL(xa_calc_term) t1,
					   A_CONS_CELL(xa_calc_term) t2);

xa_calc_set	a_make_calc_set(arena_info *arena, 
				A_CONS_CELL(xa_calc_var) set_vars,
				 A_CONS_CELL(xa_calc_var) exist_vars,
				 A_CONS_CELL(xa_calc_constraint) pred);

int		a_satisfiable_p(arena_info *arena, xa_calc_set s);

xa_calc_set	a_simplify_set (arena_info *arena, xa_calc_set s);
xa_calc_set	a_simplify_set_with_times (arena_info *arena, xa_calc_set s,
					   double *zt_to_matrix,
					   double *zt_project,
					   double *zt_to_set);

void		a_print_calc_var (FILE *fp, xa_calc_var v);
void		a_print_calc_term (FILE *fp, xa_calc_term t, int mult);
void		a_print_calc_constraint (FILE *fp, xa_calc_constraint c);
void		a_print_calc_set (FILE *fp, xa_calc_set s);


#endif
