/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
%{
#include "../config.h"

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "alpha.h"
#include "alpha_calc_parser.tab.h"

extern arena_info *parser_arena;

%}

DIGIT    [0-9]
ID       [a-zA-Z][a-zA-Z0-9]*
PUNCT    [\+\-\*\&;(){},:\|]
   
%%

{DIGIT}+ 	                {  yylval._num = atoi(yytext); 
				   return NUM; }

exist { return EXIST; }

{ID}				{  yylval._var = a_make_calc_var(parser_arena, yytext);
				   return VAR; }

{PUNCT}	        		{  return yytext[0]; }

"?" { return '?'; }

"=" { return EQ; }
"<" { return LT; }
">" { return GT; }
"<=" { return LE; }
">=" { return GE; }

"#"[^\n]*			/* eat up one-line comments */;

[ \t\n]+			/* eat up whitespace */;

.				{ fprintf (stderr, 
					   "Unrecognized text: %s\n", 
					   yytext );
			          return 0; }

%%

int yywrap(void) { return 1; }
