/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
%{
#include "../config.h"

#include <stdlib.h>
#include <assert.h>
#include "alpha.h"

int yylex();
int yyerror();
A_CONS_CELL(xa_calc_term) xa_negate_terms (A_CONS_CELL(xa_calc_term) p);

arena_info *parser_arena = 0;

%}
%union {
  int     			_num;
  xa_calc_var			_var;
  xa_calc_term			_term;
  xa_calc_constraint		_cnstr;
  xa_calc_set			_set;
  A_CONS_CELL(xa_calc_var)	_vars;
  A_CONS_CELL(xa_calc_term)	_terms;
  A_CONS_CELL(xa_calc_constraint) _pred;
}

%token        	EXIST
%token	      	EQ
%token	      	LE
%token	      	LT
%token	      	GT
%token	      	GE
%token <_num>  	NUM
%token <_var>  	VAR 
%type  <_term>  signed_term
%type  <_term>  term
%type  <_cnstr> constraint
%type  <_terms> terms
%type  <_vars>  vars
%type  <_pred>  predicate
%type  <_set>   set_expression


%right '='
%left '&'
%left '-' '+'
%left '*' '/'
%left NEG     /* Negation--unary minus */
%right '^'    /* Exponentiation        */

/* Grammar follows */

%%

input:   /* empty */
| input {
    assert(parser_arena == 0);
    parser_arena = a_arena_make(); 
} line {
    a_arena_free(parser_arena);
    parser_arena = 0; 
};
     
line:
  ';'
  | '?' set_expression ';' { xa_calc_set s2;
                             int res = a_satisfiable_p(parser_arena, $2);
                             a_print_calc_set (stdout, $2);
			     printf ("\n");
			     printf ("%s\n", res ? "true" : "false");
                             }
  | set_expression ';'  { xa_calc_set s2;
                          double t_to_matrix, t_project, t_to_set;
                          s2 = a_simplify_set_with_times(
			      parser_arena, $1,
			      &t_to_matrix, &t_project, &t_to_set);  
			  printf ("# to_matrix %f  project %f to set %f\n", 
				  t_to_matrix, t_project, t_to_set);
                          a_print_calc_set (stdout, $1);
			  printf ("\n"); 
                          a_print_calc_set (stdout, s2);
			  printf ("\n"); 
                          }
  | error ';' { yyerrok; xa_cons_count = 0; }
;

set_expression:
  '{' vars '|' EXIST vars ':' predicate '}'
  			{ $$ = a_make_calc_set (parser_arena, $2, $5, $7); }
;

vars: /*empty*/
  			{ $$ = 0; }  
  | VAR 		{ a_make_cons(parser_arena, xa_calc_var, $$, $1, 0); }
  | VAR ',' vars 	{ a_make_cons(parser_arena, xa_calc_var, $$, $1, $3); }
;

predicate: /*empty*/
  			{ $$ = 0; }
  | constraint		{ a_make_cons(parser_arena, xa_calc_constraint, $$, $1, 0); }
  | constraint '&' predicate
			{ a_make_cons(parser_arena, xa_calc_constraint, $$, $1, $3); }
;

constraint:
  terms EQ terms	{ $$ = a_make_calc_constraint(parser_arena, XA_CALC_EQ, $1, $3); }
  | terms LE terms	{ $$ = a_make_calc_constraint(parser_arena, XA_CALC_LE, $1, $3); }
  | terms LT terms	{ $$ = a_make_calc_constraint(parser_arena, XA_CALC_LT, $1, $3); }
  | terms GT terms	{ $$ = a_make_calc_constraint(parser_arena, XA_CALC_GT, $1, $3); }
  | terms GE terms	{ $$ = a_make_calc_constraint(parser_arena, XA_CALC_GE, $1, $3); }
;

terms:
  signed_term		{ a_make_cons(parser_arena, xa_calc_term, $$, $1, 0); }
  | terms '+' signed_term { a_make_cons(parser_arena, xa_calc_term, $$, $3, $1); }
  | terms '-' signed_term { xa_coeff_calc_term($3) *= -1;
                          a_make_cons(parser_arena, xa_calc_term, $$, $3, $1); }
;

signed_term:
    term { $$ = $1; }
  | '-' term %prec NEG  { xa_coeff_calc_term($2) *= -1; $$ = $2; }
;

term:
  NUM			{ $$ = a_make_calc_term(parser_arena, 0, $1); }
  | VAR			{ $$ = a_make_calc_term(parser_arena, $1, 1); }
  | NUM '*' VAR 	{ $$ = a_make_calc_term(parser_arena, $3, $1); }
;


/* End of grammar */

%%

extern int yydebug;

extern FILE *fdbg;

int main (int argc, char **argv)
{
  fdbg = stderr;

  if (argc > 1) {
    yydebug = 1;
  }
  yyparse ();
  return 0;
}

int yyerror (char *s)
{
  fprintf (stderr, "calculator error: %s\n", s);
  return 0;
}

A_CONS_CELL(xa_calc_term)
xa_negate_terms (A_CONS_CELL(xa_calc_term) p)
{
  A_FOR_CONS_CHAIN(xa_calc_term,p,q) {
    xa_coeff_calc_term(a_car(q)) *= -1;
  }
  A_END_CONS_CHAIN;

  return p;
}
