/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
/* Vladimir Kotlyar  February 1996
 *
 *
 *
 */


#ifndef _a_fmve_private_h
#define _a_fmve_private_h

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#ifndef _a_range_h
#include "alpha_range.h"
#endif

#ifndef _a_ineq_h
#include "alpha_ineq.h"
#endif

#ifndef _a_memory_h
#include "alpha_memory.h"
#endif

/***********************************************************************
 *		INFORMATION ABOUT ELIMINATED VARIABLES
 ***********************************************************************/

struct xa_elim_info_s {
  int		upper_max;
  int		lower_max;
  int		n_upper;
  int		n_lower;
};

typedef  struct xa_elim_info_s xa_elim_info_t;
typedef  struct xa_elim_info_s *xa_elim_info;

#define xa_elt_einfo(ei,v,n)		((ei)[xa_check_range((v),(n)),((v)-1)])
#define	xa_umax_einfo(ei,v,n)		(xa_elt_einfo(ei,v,n).upper_max)
#define xa_lmax_einfo(ei,v,n)		(xa_elt_einfo(ei,v,n).lower_max)
#define xa_n_upper_einfo(ei,v,n)       	(xa_elt_einfo(ei,v,n).n_upper)
#define xa_n_lower_einfo(ei,v,n)       	(xa_elt_einfo(ei,v,n).n_lower)

#define xa_n_proj_einfo(ei,v,n)	\
  (xa_n_upper_einfo(ei,v,n) * xa_n_lower_einfo(ei,v,n))

#define xa_is_exact_einfo(ei,v,n) \
  ((xa_umax_einfo(ei,v,n) == 1) || (xa_lmax_einfo(ei,v,n) == 1))

#define xa_is_unbound_einfo(ei,v,n) \
  ((xa_n_upper_einfo(ei,v,n) == 0) || (xa_n_lower_einfo(ei,v,n) == 0))

#define xa_make_einfo(arena, nvars)	\
  (a_nalloc0(arena, xa_elim_info_t, (assert((nvars)>0),(nvars))))

xa_elim_info 	xa_compile_einfo (arena_info *arena, int nvars, AlphaIneqList C);
void 		xa_elim_unbound (int nvars, AlphaIneqList C, xa_elim_info ei, int nprot);
int		xa_var2elim (int nvars, xa_elim_info ei, int nprot);


/***********************************************************************
 *		       PROJECTION CONJUNCTIONS
 ***********************************************************************/

/*
 * Here is what xa_proj type represents:
 *
 * We want to compute the set:
 *  {x | exists y : A*[x y]' == B  and C*[x y]' <= D}
 *
 * The answer we get back is a disjunction of conjunctions of the form:
 *  {x | exists w : x = x0 + H*w and F * w <= G } where dim(x) >= dim(w)             (1)
 *
 * At an intermediate stage we will have conjunctions of the form:
 *  {x | exists w = [w1 w2]' : x = x0 + H*w and F * w <= G } with dim(x) >= dim(w1)  (2)
 * 
 * We bring (2) into the form (1) by projecting away the unprotected variables w2
 * 
 * To represent one such conjunction we need:
 *   the (x0, H) pair for the "substitution"
 *   the set of inequalities F * w <= G
 *   the length of the protected part (w1)
 *
 * As a special case, we might have tautologies. They are represented by
 * setting X0 to null
 */

struct xa_proj_s {
  AlphaVector		X0;
  AlphaMatrix		H;
  AlphaIneqList		C;
  int			nvars;
  int			nprot;
  struct xa_proj_s	*next;
};

typedef struct xa_proj_s xa_proj_t;
typedef struct xa_proj_s *xa_proj;
typedef xa_proj xa_proj_list;


#define xa_X0_proj(p)			((p)->X0)
#define xa_H_proj(p)			((p)->H)
#define xa_ineq_list_proj(p)		((p)->C)
#define xa_nvars_proj(p)		((p)->nvars)
#define xa_nprot_proj(p)		((p)->nprot)
#define xa_next_proj(p)			((p)->next)

#define xa_car_proj_list(l)		((xa_proj)(l))
#define xa_cdr_proj_list(l)		((xa_proj_list)xa_next_proj(xa_car_proj_list(l)))

#define xa_add_to_acc_proj(arena,P,acc) \
{ xa_proj __add_Q = xa_make_proj_copy(arena, P);\
  xa_next_proj(__add_Q) = (acc); (acc) = (__add_Q); }

xa_proj xa_make_proj (arena_info *arena);
xa_proj xa_make_single_value_proj (arena_info *arena, AlphaVector X0);
xa_proj xa_make_hyperplane_proj (arena_info *arena, AlphaVector X0, AlphaMatrix H);
xa_proj xa_make_proj_copy (arena_info *arena, xa_proj P);
xa_proj xa_make_proj_build (arena_info *arena, 
			    int nvars, int nprot, AlphaVector X0, 
			    AlphaMatrix H, AlphaIneqList C);

void	xa_dump_proj (FILE *fp, xa_proj P);
void	xa_dump_proj_list (FILE *fp, xa_proj_list l);

extern int xa_splinter_symbolic;

#endif


