(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* Block solve clique							*)
(************************************************************************)

let bs_clique_maker args =
    let n = required an_expr_arg args "n"
    and size = required an_expr_arg args "size"
    and index = required an_expr_arg args "index"
    and values = required an_expr_var_arg args "values"
    in let base_args = [
	("$n",An_expr(n));
	("$size",An_expr(size));
	("$index",An_expr(index));
	("$values",An_string(values))
    ]
    in let enum_i_or_j_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(parse_expr "$index" base_args)
		(parse_expr "$index+$size-1" base_args)
    and lookup_v_m prereq body_f =
	let (i,j) = List'.match2 prereq
	in let args =
	    [("$i",An_expr(i));
	     ("$j",An_expr(j))]@base_args
	in
	    body_f (parse_expr "$values[[$i-$index,$j-$index]]" args)
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_i",[],"i", Am_rel(Am_rel_interval(enum_i_or_j_m)));
	 ("enum_j",[],"j", Am_rel(Am_rel_interval(enum_i_or_j_m)));
	 ("lookup_v",["i";"j"],"v",Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and  make_bounds vars =
	let n = required an_expr_arg args "n"
	and size = required an_expr_arg args "size"
	and index = required an_expr_arg args "index"
	and (i,j) = List'.match2 vars
	in
	    parse_expr
		"0 <= $i && $i <= $n-1
		 && $index <= $i && $i <= $index+$size-1
		 && 0 <= $j && $j <= $n-1
		 && $index <= $j && $j <= $index+$size-1
		 && 0 <= $index && $index <= $n-1
		 && 0 <= $size && $size <= $n
		 && $index+$size-1 <= $n-1
		 && 0 <= $n"
		[("$i",An_expr(i));("$j",An_expr(j));
		 ("$n",An_expr(n));("$size",An_expr(size));
		 ("$index",An_expr(index))]
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [];
	bb_bounds_maker = make_bounds
    }
;;

register "bs_clique_i" bs_clique_maker;;
