(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* Block solve clique							*)
(************************************************************************)

let bs_clique_maker args =
    let n = required an_expr_arg args "n"
    and size = required an_expr_arg args "size"
    and index = required an_expr_arg args "index"
    and values = required an_expr_var_arg args "values"
    in let base_args = [
	("$n",An_expr(n));
	("$size",An_expr(size));
	("$index",An_expr(index));
	("$values",An_string(values))
    ]
    in let enum_ii_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(parse_expr "0" base_args)
		(parse_expr "$size-1" base_args)
    and lookup_i_m prereq body_f =
	let (ii) = List'.match1 prereq
	in let args =
	    [("$ii",An_expr(ii))]@base_args
	in
	    body_f (parse_expr "$index + $ii" args)
    and enum_jj_m prereq body_f =
	body_f
	    (parse_expr "0" base_args)
	    (parse_expr "$size-1" base_args)
    and lookup_j_m prereq body_f =
	let (jj) = List'.match1 prereq
	in let args =
	    [("$jj",An_expr(jj))]@base_args
	in
	    body_f (parse_expr "$index + $jj" args)
    and lookup_v_m prereq body_f =
	let (ii,jj) = List'.match2 prereq
	in let args =
	    [("$ii",An_expr(ii));
	     ("$jj",An_expr(jj))]@base_args
	in
	    body_f (parse_expr "$values[[$ii,$jj]]" args)
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_ii",[],"ii", Am_rel(Am_rel_interval(enum_ii_m)));
	 ("lookup_i",["ii"],"i", Am_fun(O_1,Am_fun_lookup(lookup_i_m)));
	 ("enum_jj",[],"jj", Am_rel(Am_rel_interval(enum_jj_m)));
	 ("lookup_j",["jj"],"j", Am_fun(O_1,Am_fun_lookup(lookup_j_m)));
	 ("lookup_v",["ii";"jj"],"v", Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and make_bounds vars =
	let n = required an_expr_arg args "n"
	and size = required an_expr_arg args "size"
	and index = required an_expr_arg args "index"
	and (i,j) = List'.match2 vars
	in
	    parse_expr
		"0 <= $i && $i <= $n-1
		 && 0 <= $j && $j <= $n-1
		 && 0 <= $index && $index <= $n-1
		 && 0 <= $size && $size <= $n
		 && $index+$size-1 <= $n-1
		 && 0 <= $n"
		[("$i",An_expr(i));("$j",An_expr(j));
		 ("$n",An_expr(n));("$size",An_expr(size));
		 ("$index",An_expr(index))]
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [];
	bb_bounds_maker = make_bounds
    }
;;

register "bs_clique_ii" bs_clique_maker;;




