(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* Block solve inodes list						*)
(************************************************************************)

let bs_inodes_maker args =
    let n = required an_expr_arg args "n"
    and num_inodes = required an_expr_arg args "num_inodes"
    and num_row_indices = required an_expr_arg args "num_row_indices"
    and num_vals = required an_expr_arg args "num_vals"
    and num_rows = required an_expr_var_arg args "num_rows"
    and num_cols = required an_expr_var_arg args "num_cols"
    and col_indices = required an_expr_var_arg args "col_indices"
    and row_ptr = required an_expr_var_arg args "row_ptr"
    and row_indices = required an_expr_var_arg args "row_indices"
    and val_ptr = required an_expr_var_arg args "val_ptr"
    and values = required an_expr_var_arg args "values"
    in let base_args = [
	("$n",An_expr(n));
	("$num_inodes",An_expr(num_inodes));
	("$num_row_indices",An_expr(num_row_indices));
	("$num_vals",An_expr(num_vals));
	("$num_rows",An_string(num_rows));
	("$num_cols",An_string(num_cols));
	("$col_indices",An_string(col_indices));
	("$row_ptr",An_string(row_ptr));
	("$row_indices",An_string(row_indices));
	("$val_ptr",An_string(val_ptr));
	("$values",An_string(values))
    ]
    in let enum_ind_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(parse_expr "0" base_args)
		(parse_expr "$num_inodes-1" base_args)
    and enum_ii_m prereq body_f =
	let (ind) = List'.match1 prereq
	in let args = [("$ind",An_expr(ind))]@base_args
	in
	    body_f
		(parse_expr "0" args)
		(parse_expr "$num_rows[[$ind]]-1" args)
    and lookup_i_m prereq body_f =
	let (ind,ii) = List'.match2 prereq
	in let args =
	    [("$ind",An_expr(ind));("$ii",An_expr(ii))]@base_args
	in
	    body_f (parse_expr "$row_indices[[$row_ptr[[$ind]] + $ii]]"
			args)
    and enum_jj_m prereq body_f =
	let (ind) = List'.match1 prereq
	in let args = [("$ind",An_expr(ind))]@base_args
	in
	    body_f
		(parse_expr "0" args)
		(parse_expr "$num_cols[[$ind]]-1" args)
    and lookup_j_m prereq body_f =
	let (ind,jj) = List'.match2 prereq
	in let args =
	    [("$ind",An_expr(ind));("$jj",An_expr(jj))]@base_args
	in
	    body_f (parse_expr "$col_indices[[$ind]] + $jj" args)
    and lookup_v_m prereq body_f =
	let (ind,ii,jj) = List'.match3 prereq
	in let args =
	    [("$ind",An_expr(ind));
	     ("$ii",An_expr(ii));
	     ("$jj",An_expr(jj))]@base_args
	in let inode = Misc.gensym "inode"
	in let args' = ("$inode",An_string(inode))::args
	in let body = body_f (parse_expr "$inode[[$ii,$jj]]" args')
	in let args'' = ("$body",An_stmt(body))::args'
	in
	    parse_stmt "{ 
		var $inode :
		    array (0:$num_rows[[$ind]]-1,
			   0:$num_cols[[$ind]]-1)
			of real
			:= &$values[[ $val_ptr[[$ind]] ]];
		$body;
	    }" args''
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_ind",[],"ind", Am_rel(Am_rel_interval(enum_ind_m)));
	 ("enum_ii",["ind"],"ii", Am_rel(Am_rel_interval(enum_ii_m)));
	 ("lookup_i",["ind";"ii"],"i",
	    Am_fun(O_1,Am_fun_lookup(lookup_i_m)));
	 ("enum_jj",["ind"],"jj", Am_rel(Am_rel_interval(enum_jj_m)));
	 ("lookup_j",["ind";"jj"],"j",
	    Am_fun(O_1,Am_fun_lookup(lookup_j_m)));
	 ("lookup_v",["ind";"ii";"jj"],"v",
	    Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and make_bounds = rectangular_bounds0 "n" "n" args
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [];
	bb_bounds_maker = make_bounds
    }
;;

register "bs_inodes" bs_inodes_maker;;




