(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* Compressed compressed row storage					*)
(************************************************************************)

let ccrs_maker args =
    let n = required an_expr_arg args "n"
    and nrs = required an_expr_arg args "nrs"
    and rowind_ptr = required an_expr_var_arg args "rowind"
    and rowptr_ptr = required an_expr_var_arg args "rowptr"
    and nzs = required an_expr_arg args "nzs"
    and colind_ptr = required an_expr_var_arg args "colind"
    and value_ptr = required an_expr_var_arg args "value"
    in let enum_ii_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f (Expr_int(0)) (Expr_op(Op_plus,[nrs;Expr_int(-1)]))
    and lookup_i_m prereq body_f =
	let ii = List'.match1 prereq
	in
	    body_f (parse_expr "$rindp[[$ii]]"
			[("$rindp",An_string(rowind_ptr));
			 ("$ii",An_expr(ii))])
    and search_i_m prereq found_f not_found_f =
	let i = List'.match1 prereq
	in
	    gen_binary_search i (Expr_int(0)) nrs rowind_ptr
		found_f not_found_f
    and enum_jj_m prereq body_f =
	let (ii) = List'.match1 prereq
	in let args = [
	    ("$rowp",An_string(rowptr_ptr));
	    ("$ii",An_expr(ii))
	] in
	    body_f
		(parse_expr "$rowp[[$ii]]" args)
		(parse_expr "$rowp[[$ii+1]]-1" args)
    and lookup_j_m prereq body_f =
	let (ii,jj) = List'.match2 prereq
	in
	    body_f (parse_expr "$cindp[[$jj]]"
			[("$cindp",An_string(colind_ptr));
			 ("$jj",An_expr(jj))])
    and search_j_m prereq found_f not_found_f =
	let (ii,j) = List'.match2 prereq
	in let parser_args =
	    [("$rowp",An_string(rowptr_ptr)); ("$ii",An_expr(ii))]
	in
	    gen_binary_search j
		(parse_expr "$rowp[[$ii]]" parser_args)
		(parse_expr "$rowp[[$ii+1]]-1" parser_args)
		colind_ptr found_f not_found_f
    and lookup_v_m prereq body_f =
	let (ii,jj) = List'.match2 prereq
	in
	    body_f (parse_expr "$valp[[$jj]]"
			[("$valp",An_string(value_ptr));
			 ("$jj",An_expr(jj))])
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_ii",[],"ii", Am_rel(Am_rel_interval(enum_ii_m)));
	 ("lookup_i",["ii"],"i", Am_fun(O_1,Am_fun_lookup(lookup_i_m)));
	 ("search_i",["i"],"ii", Am_fun(O_logn,Am_fun_search(search_i_m)));
	 ("enum_jj",["ii"],"jj", Am_rel(Am_rel_interval(enum_jj_m)));
	 ("lookup_j",["ii";"jj"],"j",
	    Am_fun(O_1,Am_fun_lookup(lookup_j_m)));
	 ("search_j",["ii";"j"],"jj",
	    Am_fun(O_logn,Am_fun_search(search_j_m)));
	 ("lookup_v",["ii";"jj"],"v",
	    Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and make_bounds =  rectangular_bounds0 "n" "n" args
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [("enum_ii","lookup_i"); ("enum_jj","lookup_j")]; 
	bb_bounds_maker = make_bounds
    }
;;

register "ccrs" ccrs_maker;;
