(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* Coordinate storage							*)
(************************************************************************)

let coor_maker args =
    let n = required an_expr_arg args "n"
    and nzs = required an_expr_arg args "nzs"
    and rowind_ptr = required an_expr_var_arg args "rowind"
    and colind_ptr = required an_expr_var_arg args "colind"
    and value_ptr = required an_expr_var_arg args "value"
    in let enum_k_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(Expr_int(0))
		(Expr_op(Op_plus,[nzs;Expr_int(-1)]))
    and lookup_i_m prereq body_f =
	let k = List'.match1 prereq
	in
	    body_f (parse_expr "$rowind[[$k]]"
			[("$rowind",An_string(rowind_ptr));
			 ("$k",An_expr(k))])
    and lookup_j_m prereq body_f =
	let k = List'.match1 prereq
	in
	    body_f (parse_expr "$colind[[$k]]"
			[("$colind",An_string(colind_ptr));
			 ("$k",An_expr(k))])
    and lookup_v_m prereq body_f =
	let k = List'.match1 prereq
	in
	    body_f (parse_expr "$valp[[$k]]"
			[("$valp",An_string(value_ptr));
			 ("$k",An_expr(k))])
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_k", [], "k", Am_rel(Am_rel_interval(enum_k_m)));
	 ("lookup_v", ["k"], "v", Am_fun(O_1,Am_fun_lookup(lookup_v_m)));
	 ("lookup_i", ["k"], "i", Am_fun(O_1,Am_fun_lookup(lookup_i_m)));
	 ("lookup_j", ["k"], "j", Am_fun(O_1,Am_fun_lookup(lookup_j_m)))
	]
    and make_bounds = rectangular_bounds0 "n" "n" args
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [];
	bb_bounds_maker = make_bounds
    }
;;

register "coor" coor_maker;;
