(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl
open Containers

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

let string_to_triang_flags str =
  let str' = String.lowercase str
  and string_mem s c =
    try String.index s c; true with Not_found -> false
  in let lower_p = string_mem str' ('l')
  and diag_p = string_mem str' ('d')
  and upper_p = string_mem str' ('u')
  in (lower_p,diag_p,upper_p)

(************************************************************************)
(* Compressed row storage						*)
(************************************************************************)

let crs_maker args =
    let n = required an_expr_arg args "n"
    and rowptr_ptr = required an_expr_var_arg args "rowptr"
    and nzs = required an_expr_arg args "nzs"
    and colind_ptr = required an_expr_var_arg args "colind"
    and value_ptr = required an_expr_var_arg args "value"
    and (lower_p,diag_p,upper_p) =
      string_to_triang_flags
	(an_string_arg args "store" (fun () -> "ldu"))
    in let enum_i_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(Expr_int(0))
		(Expr_op(Op_plus,[n;Expr_int(-1)]))
    and enum_jj_m prereq body_f =
	let i = List'.match1 prereq
	in let args = [
	    ("$rowp",An_string(rowptr_ptr));
	    ("$i",An_expr(i))
	] in
	    body_f
		(parse_expr "$rowp[[$i]]" args)
		(parse_expr "$rowp[[$i+1]]-1" args)
    and lookup_j_m prereq body_f =
	let (i,jj) = List'.match2 prereq
	in
	    body_f (parse_expr "$cindp[[$jj]]"
			[("$cindp",An_string(colind_ptr));
			 ("$jj",An_expr(jj))])
    and search_j_m prereq found_f not_found_f =
	let (i,j) = List'.match2 prereq
	in let parser_args =
	    [("$rowp",An_string(rowptr_ptr)); ("$i",An_expr(i))]
	in
	    gen_binary_search j
		(parse_expr "$rowp[[$i]]" parser_args)
		(parse_expr "$rowp[[$i+1]]-1" parser_args)
		colind_ptr found_f not_found_f
    and lookup_v_m prereq body_f =
	let (i,jj) = List'.match2 prereq
	in
	    body_f (parse_expr "$valp[[$jj]]"
			[("$valp",An_string(value_ptr));
			 ("$jj",An_expr(jj))])
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_i",[],"i", Am_rel(Am_rel_interval(enum_i_m)));
	 ("enum_jj",["i"],"jj", Am_rel(Am_rel_interval(enum_jj_m)));
	 ("lookup_j",["i";"jj"],"j",
	    Am_fun(O_1,Am_fun_lookup(lookup_j_m)));
	 ("search_j",["i";"j"],"jj",
	    Am_fun(O_logn,Am_fun_search(search_j_m)));
	 ("lookup_v",["i";"jj"],"v",
	    Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and make_bounds = triangular_bounds n n lower_p diag_p upper_p
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [("enum_jj","lookup_j")];
	bb_bounds_maker = make_bounds
    }
;;

register "crs" crs_maker;;

(************************************************************************)
(* Compressed column storage						*)
(************************************************************************)

let ccs_maker args =
  let colptr_ptr = required an_expr_var_arg args "colptr"
  and rowind_ptr = required an_expr_var_arg args "rowind"
    and (lower_p,diag_p,upper_p) =
      string_to_triang_flags
	(an_string_arg args "store" (fun () -> "ldu"))
  in let store_str' =
    (if lower_p then "u" else "")
      ^(if diag_p then "d" else "")
      ^(if upper_p then "l" else "")
  in let args' =
    String_map.add "rowptr" (An_expr(Expr_var(colptr_ptr)))
      (String_map.add "colind" (An_expr(Expr_var(rowind_ptr)))
	 (String_map.add "store_str" (An_expr(Expr_string(store_str')))
	    args))
  in let { bb_index_fields = ind_fields;
    	bb_value_field = val_field;
    	bb_access_methods = acc_mthds;
    	bb_ordered_access = ordered_acc;
    	bb_bounds_maker = bounds;
      } = crs_maker args'
  in let ind_fields' = List.rev ind_fields;
  in
    { bb_index_fields = ind_fields';
      bb_value_field = val_field;
      bb_access_methods = acc_mthds;
      bb_ordered_access = ordered_acc;
      bb_bounds_maker = bounds;
    }
;;

register "ccs" ccs_maker;;
