(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* ITPACK/ELLPACK							*)
(************************************************************************)

let itpack_maker args =
    let n = required an_expr_arg args "n"
    and width = required an_expr_arg args "width"
    and colind = required an_expr_var_arg args "colind"
    and values = required an_expr_var_arg args "values"
    in let enum_jj_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(Expr_int(0))
		(Expr_op(Op_plus,[width;Expr_int(-1)]))
    and enum_i_m prereq =
	let jj = List'.match1 prereq
	in let i = Misc.gensym "i"
	in let i_ident =
	    Ident_valid(i,Type_scalar(Type_int),Ident_var(Expr_void),
		An_list([]))
	in let parse_args = [
	    ("$jj",An_expr(jj));
	    ("$i",An_string(i));
	    ("$n",An_expr(n));
	    ("$colind",An_string(colind))
	] in let parse_args' = ("$advance",An_stmt(
		parse_stmt "
		    while ($i < $n && $colind[[ $jj, $i ]] == -1) {
			$i := $i + 1;
		    }" parse_args
	    ))::parse_args
	in
	    { decl_ids = [ i_ident ];
	      init_st = parse_stmt "{ $i := 0; $advance; }"
			    parse_args';
	      incr_st = parse_stmt "{ $i := $i + 1; $advance; }"
			    parse_args';
	      close_st = Stmt_nop;
	      valid_ex = parse_expr "$i < $n" parse_args';
	      deref_ex = parse_expr "$i" parse_args'
	    }
    and lookup_j_m prereq body_f =
	let (jj,i) = List'.match2 prereq
	in
	    body_f (parse_expr "$colind[[$jj,$i]]"
			[("$colind",An_string(colind));
			 ("$jj",An_expr(jj));
			 ("$i",An_expr(i))])
    and lookup_v_m prereq body_f =
	let (jj,i) = List'.match2 prereq
	in
	    body_f (parse_expr "$values[[$jj,$i]]"
			[("$values",An_string(values));
			 ("$jj",An_expr(jj));
			 ("$i",An_expr(i))])
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_jj",[],"jj", Am_rel(Am_rel_interval(enum_jj_m)));
	 ("enum_i",["jj"],"i", Am_rel(Am_rel_general(enum_i_m)));
	 ("lookup_j_m",["jj";"i"],"j",
	    Am_fun(O_1,Am_fun_lookup(lookup_j_m)));
	 ("lookup_v_m",["jj";"i"],"v",
	    Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and make_bounds = rectangular_bounds0 "n" "n" args
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [];
	bb_bounds_maker = make_bounds
    }
;;

register "itpack" itpack_maker;;
