(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

open Bmf;;
open Bmf_parser_aux;;
open Bb;;
open Bb_utils;;

(************************************************************************)
(* Jagged Diagonal							*)
(************************************************************************)

let jdiag_maker args =
    let n = required an_expr_arg args "n"
    and nzs = required an_expr_arg args "nzs"
    and width = required an_expr_arg args "width"
    and diagptr = required an_expr_var_arg args "diagptr"
    and colind = required an_expr_var_arg args "colind"
    and values = required an_expr_var_arg args "values"
    in let enum_jj_m prereq body_f =
	let _ = List'.match0 prereq
	in
	    body_f
		(Expr_int(0))
		(Expr_op(Op_plus,[width;Expr_int(-1)]))
    and enum_kk_m prereq body_f =
	let (jj) = List'.match1 prereq
	in let parse_args =[
	    ("$jj",An_expr(jj));
	    ("$diagptr",An_string(diagptr))
	] in
	    body_f
		(parse_expr "$diagptr[[$jj]]" parse_args)
		(parse_expr "$diagptr[[$jj+1]]-1" parse_args)
    and lookup_i_m prereq body_f =
	let (jj,kk) = List'.match2 prereq
	in let parse_args =[
	    ("$jj",An_expr(jj));
	    ("$kk",An_expr(kk));
	    ("$diagptr",An_string(diagptr))
	] in
	    body_f 
		(parse_expr "$kk-$diagptr[[$jj]]" parse_args)
    and lookup_j_m prereq body_f =
	let (jj,kk) = List'.match2 prereq
	in let parse_args =[
	    ("$kk",An_expr(kk));
	    ("$colind",An_string(colind))
	] in
	    body_f 
		(parse_expr "$colind[[$kk]]" parse_args)
    and lookup_v_m prereq body_f =
	let (jj,kk) = List'.match2 prereq
	in let parse_args =[
	    ("$kk",An_expr(kk));
	    ("$values",An_string(values))
	] in
	    body_f 
		(parse_expr "$values[[$kk]]" parse_args)
    in let acc_mthd (name,in_fs,out_f,info) = {
	am_name = name; am_in_fields = in_fs;
	am_out_field = out_f; am_info = info
    } in let acc_mthds =
	[
	 ("enum_jj",[],"jj", Am_rel(Am_rel_interval(enum_jj_m)));
	 ("enum_kk",["jj"],"kk", Am_rel(Am_rel_interval(enum_kk_m)));
	 ("lookup_i_m",["jj";"kk"],"i",
	    Am_fun(O_1,Am_fun_lookup(lookup_i_m)));
	 ("lookup_j_m",["jj";"kk"],"j",
	    Am_fun(O_1,Am_fun_lookup(lookup_j_m)));
	 ("lookup_v_m",["jj";"kk"],"v",
	    Am_fun(O_1,Am_fun_lookup(lookup_v_m)))
	]
    and make_bounds = rectangular_bounds0 "n" "n" args
    in {
	bb_index_fields = ["i";"j"];
	bb_value_field = "v";
	bb_access_methods = List.map acc_mthd acc_mthds;
	bb_ordered_access = [];
	bb_bounds_maker = make_bounds
    }
;;

register "jdiag" jdiag_maker;;

