(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bmf
open Bb

(* For the purposes of this interface file, a "bounds set" is an expression
   that represents some bounds (not necessarily finite) of a set of index
   variables. The following is an example of a bounds set:

   { [i,j] | 1 <= i && i <= n && 1 <= j && j <= n }

   Bounds sets are represented with Expr_set's. *)


(* The number of dimensions in an array type *)
val array_dimen : array_type -> int;;

(* The base type of an array. *)
val array_base_type : array_type -> scalar_type;;

(* The bounds of an array. The result is a bounds set. *)
val array_bounds : array_type -> bmf_expr;;

(* Strength testing. The parameter is a bounds set and, if the result is
   true, then the array is guaranteed to have an allocated location for
   every index within this bounds set. This function may throw an Failure
   exception if the parameter bounds set is not a subset of the arrays
   bounds. *)

val array_strength : array_type -> bmf_expr -> bool;;

(* Other routines. *)

val is_sparse_array : bmf_ident -> bool
