(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Alpha;;
open Ssisl
open Containers
open Bmf;;

(*
 * affine form consists of a map from ident refs to ints (coefficients)
 * plus the constant part
 *)
type
  affine_form =
    Form                 of (int String_map.t) * int
  | Non_linear

and
  affine_term = 
    Term                 of string * int

and
  affine_map =
    Affine_map           of matrix * vector
;;

exception Non_linear_form of unit;;
exception Non_conjunct of bmf_expr;;
exception Bad_affine_map of ((bmf_ident list) * affine_map );;
exception Bad_affine_rel of ((bmf_ident list) * affine_map * (bmf_ident list) * affine_map);;
exception Zero_term of unit;;

val affine_map_rows : affine_map -> int;;
val affine_map_cols : affine_map -> int;;
val new_affine_map : int -> int -> affine_map;;
val compose_affine_map : affine_map -> affine_map -> affine_map;;
val is_zero_affine_map : affine_map -> bool;;
val is_ident_affine_map : affine_map -> bool;;
val new_affine_form : affine_term_builder -> affine_form;;
val get_const4affine_form : affine_form -> int;;
val get_term_list : affine_form -> affine_term list;;
val collect_vars : affine_form -> string list;;
val affine_add : affine_form -> affine_form -> affine_form;;
val affine_saxpy : int -> affine_form -> int -> affine_form -> affine_form;;
val affine_scale : int -> affine_form -> affine_form;;
val affine_mult : affine_form -> affine_form -> affine_form;;
val affine_coeff4var : string -> affine_form -> int;;
val affine_scale_by_list : int -> int -> affine_term list -> affine_form;;
val affine_add_by_list : int -> affine_term list -> affine_form ->
    affine_form;;
val expr2affine : bmf_expr -> affine_form;;
val affine2expr : affine_form -> bmf_expr;;
val is_linear_form : affine_form -> bool;;
val term_list2vector : string list -> affine_term list -> int array ref ->
    unit;;
val affine2vector : string list -> affine_form -> int array;;
val affine_list2affine_map : string list -> affine_form list -> 
    affine_map;;
val verify_affine_map : string list -> affine_map -> affine_map;;
val affine_map2affine_list : string list -> affine_map -> 
    affine_form list;;
val expr_list2affine_map : string list -> bmf_expr list -> affine_map;;
val affine_relation2expr_list :
 string list -> affine_map -> string list -> affine_map -> bmf_expr list;;
val affine_map2string : affine_map -> string;;
val create_identity_affine_map : int -> affine_map;;
val extract_rows_affine_map : affine_map -> int -> int -> affine_map;;
val extend_affine_maps : affine_map -> affine_map -> 
    affine_map * affine_map;;
val stack_affine_maps : affine_map -> affine_map -> affine_map;;
val compare_affine_maps : affine_map -> affine_map -> int;;
val equal_affine_map : affine_map -> affine_map -> bool;;
val destructure_linear_map : affine_map -> matrix;;
val destructure_affine_map :
 affine_map -> ((int * int) * int array array) * (int * int array);;
val make_linear_map : matrix -> affine_map;;

exception Invalid_expr_set of bmf_expr;;

val expr_set2builder_set : bmf_expr -> affine_set_builder;;
val builder_set2expr_set : affine_set_builder -> bmf_expr;;

exception Equalities_not_allowed of bmf_expr;;

exception Incompatible_sets;;

val intersect_sets : bmf_expr -> bmf_expr -> bmf_expr;;
val simplify_set : bmf_expr -> bmf_expr;;
val gist_sets : bmf_expr -> bmf_expr -> bmf_expr;;

val fm_code_gen : 
    bmf_expr (* Set expr returned by alpha__simplify *) -> 
    ( bmf_expr list (* lower bounds *) *
      bmf_expr list (* upper bounds *) ) list (* for each var *);;
