(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bmf;;

(************************************************************************)
(* Querys. Representation and manipulation				*)
(************************************************************************)

(* A SELECT-FROM-WHERE query. 
   Relations model pseudo-dense arrays. That is, arrays with references for 
     all indices an their bounds, but presence bits to indicate when value
     are stored and not stored. *)
type query = {
    (* The identifiers that are bound in the query. These are either
       qb_value or qb_flag names or loop index names. *)
    query_vars: (string * query_var_kind) list;
    (* The list of names that used in the body of the query.
       This corrisponds to the SELECT clause. 
       If there are names in query_vars that do not appear in query_used,
       then a projection is to be performed. *)
    query_used: string list;
    (* The relations that are used in the query.
       This corrisponds to the FROM clause. *)
    query_bindings: query_binding list;
    (* The original loop bounds.
       These implicitly define an iteration relation, I, that is also part
       of the FROM clause. *)
    query_bounds: bmf_expr;
    (* The sparsity predicate.
       This corrisponds to the WHERE clause. *)
    query_pred: Wff.Simple.t;
    (* predicates that are true, REGARDLESS OF THE BOUNDS OF SPARSITY
       PREDICATES!
       To assert that a reference, x, is strong, use the following
       implementation: query_pred => x.
       To indicate that a reference is pre-allocated based upon a query's
       predicate, use the following equality: query_pred = x. *)
    query_assertion: Wff.Simple.t;
    (* The body of the query. *)
    query_body: bmf_stmt
}
and query_var_kind =
      Qv_loop
    | Qv_value of scalar_type
    | Qv_flag

and query_binding = {
    qb_rel: string;
    qb_acc: bmf_expr list;
    qb_value: string;
    qb_flag: string;
}
;;

exception Not_a_valid_query;;

val stmt_is_query: bmf_stmt -> bool;;
val stmt2query: bmf_stmt -> query;;
val query2stmt: query -> bmf_stmt;;
val normalize_query : query -> query;;

val collect_loop_vars : query -> string list;;
val is_pred_strong_p : query -> string -> bool
val is_assert_strong_p : query -> string -> bool
