(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl

val an_get_string : Bmf.annotation -> string
val an_get_list : Bmf.annotation -> Bmf.annotation list
val an_get_string_list : Bmf.annotation -> string list
val string_list_2_an_list : string list -> Bmf.annotation
val collect_ann_4_ident : Bmf.bmf_ident -> Bmf.annotations
val is_valid_ident : Bmf.bmf_ident -> bool
val is_array : Bmf.bmf_ident -> bool
val array_elt_type : Bmf.bmf_ident -> Bmf.scalar_type
val an_list2index_list : Bmf.annotation -> Bmf.bmf_ident list
val an_list2expr_list : Bmf.annotation -> Bmf.bmf_expr list
val expr_list2an_list : Bmf.bmf_expr list -> Bmf.annotation
val var_list2an_list : Bmf.bmf_ident list -> Bmf.annotation

val compact_seq_stmt : Bmf.bmf_stmt -> Bmf.bmf_stmt
val stmt_list2seq : Bmf.bmf_stmt list -> Bmf.bmf_stmt
val seq2stmt_list : Bmf.bmf_stmt -> Bmf.bmf_stmt list

(* Construct a function that maps a stmt to its parent. 
   Throws Root_stmt for the topmost stmt. 
   Throws Not_found if not within tree. *)
exception Root_stmt
val parent_of_stmt : Bmf.bmf_stmt -> Bmf.bmf_stmt -> Bmf.bmf_stmt

val collect_var_names : Bmf.bmf_expr list -> string list
val collect_vars :
  Bmf.bmf_ident list -> Bmf.bmf_expr list -> Bmf.bmf_ident list
val collect_op_args : Bmf.bmf_op -> Bmf.bmf_expr list -> Bmf.bmf_expr list
val uses_var : string -> Bmf.bmf_expr -> bool
exception Expr_diff of int
val compare_expr_list :
  (Bmf.bmf_expr -> Bmf.bmf_expr -> int) ->
  Bmf.bmf_expr list -> Bmf.bmf_expr list -> int
val compare_expr :
  (Bmf.bmf_expr -> Bmf.bmf_expr -> int) ->
  Bmf.bmf_expr -> Bmf.bmf_expr -> int
val compare_access : bool -> Bmf.bmf_expr -> Bmf.bmf_expr -> int
val compare_expr_just_access : Bmf.bmf_expr -> Bmf.bmf_expr -> int
val compare_expr_strict : Bmf.bmf_expr -> Bmf.bmf_expr -> int
val is_subexpr : ('a -> Bmf.bmf_expr -> int) -> 'a -> Bmf.bmf_expr -> bool
val collect_accesses : Bmf.bmf_expr list -> Bmf.bmf_expr list

val rename_in_expr :
  string list -> string list -> Bmf.bmf_expr -> Bmf.bmf_expr

val verify_scope : Bmf.bmf_ident list -> string -> Bmf.bmf_ident list

exception Not_a_wff of Bmf.bmf_expr
val wff_to_bmf : Wff.Simple.t -> Bmf.bmf_expr
val bmf_to_wff : Bmf.bmf_expr -> Wff.Simple.t

val uniquely_rename_within_file : Bmf.bmf_ident list -> Bmf.bmf_ident list
val uniquely_rename_within_proc : Bmf.bmf_ident -> Bmf.bmf_ident

val bmf_expr_simplify : Bmf.bmf_expr -> Bmf.bmf_expr

(* Finding free variables in various forms. *)
val ident_fvs : Bmf.bmf_ident -> Containers.String_set.t;;
val idents_fvs : Bmf.bmf_ident list -> Containers.String_set.t;;
val stmt_fvs : Bmf.bmf_stmt -> Containers.String_set.t;;
val expr_fvs : Bmf.bmf_expr -> Containers.String_set.t;;
val type_fvs : Bmf.bmf_type -> Containers.String_set.t;;
val ann_fvs : Bmf.annotation -> Containers.String_set.t;;

(* *****************************************************************
   Substitutions.

   The following two routines provide a mechanism for performing variable
   substitutions in stmt ast's.

   The placement of substitution information is seperated from the actual
   substituion so that users of these routines do not have worry about
   carrying scoping and renaming information around.

   That is, place_substitutions can be used to insert annotations
   containing substitution information into an ast. Once the ast has been
   completely transformed, perform_substitutions traverses a prodedure and
   performs all of the substitutions that specified by these annotations.

   ***************************************************************** *)

(* Place annotations for substitutions.

   pairs - list of (x,y) substitution pairs.
   body - stmt in which the substitutions are to be done. *)
val place_substitutions : 
    (string * Bmf.bmf_expr) list -> Bmf.bmf_stmt -> Bmf.bmf_stmt

(* Perform all substitutions, [x |-> y] found within a procedure or stmt
   ast. *)
val perform_substitutions_in_proc : Bmf.bmf_ident -> Bmf.bmf_ident
val perform_substitutions_in_stmt : Bmf.bmf_stmt -> Bmf.bmf_stmt
