(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
(* Case-sensitive Scheme *)
module Scheme : Sim.S

module type Sx_type =
  sig
    type t =
        Nil
      |	Bool of bool
      | Char of char
      | Float of float
      | Int of int
      | String of string
      | Symbol of string
      | Pair of t * t
      | Vector of t array
      | Other of Scheme.t
    exception Syntax_error
    val from_scheme : Scheme.t -> t
    val to_scheme : t -> Scheme.t
    val from_list : t list -> t
    val to_list : t -> t list
    val list_p : t -> bool
    val untag_symbol : t -> string
    val untag_symbol_list : t -> string list
    val tag_symbol : string -> t
    val tag_symbol_list : string list -> t
    val read : unit -> t
    val write : t -> unit 
    val pretty_print : t -> unit
    val to_string : t -> string
    val to_string_pretty : t -> string
    val define_prim0 : Scheme.env_type -> string -> (unit -> t) -> unit
    val define_prim1 : Scheme.env_type -> string -> (t -> t) -> unit
    val define_prim2 : Scheme.env_type -> string -> (t -> t -> t) -> unit
    val define_prim3 : Scheme.env_type -> string -> (t -> t -> t -> t) -> unit
    val define_prim4 : 
	Scheme.env_type -> string -> (t -> t -> t -> t -> t) -> unit
    val define_prim5 : 
	Scheme.env_type -> string -> (t -> t -> t -> t -> t -> t) -> unit
    val define_primn : Scheme.env_type -> string -> (t -> t) -> unit
  end
module Sx : Sx_type
	  


(* Atoms *)

module type AtomType =
  sig
    type t
    val compare: t -> t -> int
    exception Invalid_s_expr of Sx.t
    val from_sexpr: Sx.t -> t
    val to_sexpr: t -> Sx.t
  end

module SimpleAtom : AtomType with type t = string
