(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bmf;;
open Bmf_util;;
open Dense_walk;;
open Dense_misc;;

let rec optimize_procs blasing_p top_decls =
    (* First, simplify expressions. *)
    let top_decls =
	List.map simplify_exprs top_decls
    (* Second, do alpha renaming. *)
    in let top_decls =
	uniquely_rename_within_file top_decls
    in let top_decls =
	List.map Dense_fm.do_fm top_decls
    in let top_decls =
	List.map remove_let_decls_in_proc top_decls
    in let top_decls =
	List.map Dense_invars.hoist_let_bindings top_decls
    in let top_decls =
	List.map Dense_reindex.do_reindex top_decls
    in let top_decls =
	List.map Dense_reptrs.do_reptrs top_decls
    in let top_decls =
	List.map (Dense_gathers.do_gathers blasing_p) top_decls
    in let top_decls =
	List.map Dense_invars.hoist_let_bindings top_decls
    (* simplify expressions. *)
    in let top_decls =
	List.map simplify_exprs top_decls
    (* And finally, do blas *)
    in let top_decls =
      if blasing_p then
	Dense_blas.find_blas_in_procs top_decls
      else
	top_decls
    in
	top_decls

and simplify_exprs id =
    ident_of_walk_kind
	(map_bmf
	    (fun ctx form ->
		match form with
		  Walk_expr(ex) ->
		    Mba_return(Walk_expr(bmf_expr_simplify ex))
		| _ -> Mba_visit([WALK_ALL],ctx,(fun x -> Mba_return x)))
	    () (Walk_ident(id)))

;;

(************************************************************************)
(*			  Command line processing			*)
(************************************************************************)

let _ =
  register_switch "-dense"
    "-dense (bmf) -- Dense optimizations."
    (fun top_decls args ->
      let name = Bsct_toplevel.get_arg_name (List.hd args)
      and (options,param,args') = Bsct_toplevel.parse_arg false args
      in let (raw_for_blas_p,options) = 
	Bsct_toplevel.extract_option options 'b'
      in let (blas_p,options) = Bsct_toplevel.extract_option options 'B'
      in let for_blas_p = raw_for_blas_p || blas_p
      in let _ = Bsct_toplevel.ignore_options options name
      in let top_decls' = optimize_procs for_blas_p top_decls
      in (top_decls',args'))
