(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bmf;;

(************************************************************************)
(* A General Program Transformer					*)
(************************************************************************)

type walk_kind =
    Walk_stmt of bmf_stmt
|   Walk_expr of bmf_expr
|   Walk_type of bmf_type
|   Walk_ident of bmf_ident
|   Walk_ann of annotation
;;

type walk_what =
    WALK_ALL
|   WALK_STMTS
|   WALK_EXPRS
|   WALK_TYPES
|   WALK_IDENTS
|   WALK_ANNS
;;

val stmt_of_walk_kind : walk_kind -> bmf_stmt;;
val expr_of_walk_kind : walk_kind -> bmf_expr;;
val type_of_walk_kind : walk_kind -> bmf_type;;
val ident_of_walk_kind : walk_kind -> bmf_ident;;
val ann_of_walk_kind : walk_kind -> annotation;;

type ('a,'b) transform_bmf_action =
    Tba_return of 'b * walk_kind 
|   Tba_recurs of 'a
		* walk_kind
	        * ('b -> walk_kind -> ('a,'b) transform_bmf_action)
|   Tba_visit of walk_what list
	        * 'a
		* ('b list -> walk_kind -> ('a,'b) transform_bmf_action)
;;

val transform_bmf : ('a -> walk_kind -> ('a,'b) transform_bmf_action)
	-> 'a -> walk_kind -> ('b * walk_kind);;

type 'a map_bmf_action =
    Mba_return of walk_kind
|   Mba_recurs of 'a
		* walk_kind
	        * (walk_kind -> 'a map_bmf_action)
|   Mba_visit of walk_what list
	        * 'a
		* (walk_kind -> 'a map_bmf_action)
;;

val map_bmf : ('a -> walk_kind -> 'a map_bmf_action) ->
	'a -> walk_kind -> walk_kind;;

type ('a,'b) it_bmf_action =
    Iba_return of 'b
|   Iba_recurs of 'a
		* walk_kind
	        * ('b -> ('a,'b) it_bmf_action)
|   Iba_visit of walk_what list
	        * 'a
		* ('b list -> ('a,'b) it_bmf_action)
;;

val it_bmf : ('a -> walk_kind -> ('a,'b) it_bmf_action) ->
	'a -> walk_kind -> 'b;;

val transform_bmf_ident :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action) -> 'a -> bmf_ident
    -> 'b * bmf_ident;;

val transform_bmf_stmt :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action) -> 'a -> bmf_stmt
    -> 'b * bmf_stmt;;

val transform_bmf_expr :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action) -> 'a -> bmf_expr
    -> 'b * bmf_expr;;

val transform_bmf_type :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action) -> 'a -> bmf_type
    -> 'b * bmf_type;;

val transform_annotation :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action) -> 'a -> annotation
    -> 'b * annotation;;

val transform_bmf_idents :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action)
    -> 'a -> ('b -> 'b -> 'b) -> 'b -> bmf_ident list
    -> 'b * bmf_ident list;;

val transform_bmf_stmts :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action)
    -> 'a -> ('b -> 'b -> 'b) -> 'b -> bmf_stmt list
    -> 'b * bmf_stmt list;;

val transform_bmf_exprs :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action)
    -> 'a -> ('b -> 'b -> 'b) -> 'b -> bmf_expr list
    -> 'b * bmf_expr list;;

val transform_bmf_types :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action)
    -> 'a -> ('b -> 'b -> 'b) -> 'b -> bmf_type list
    -> 'b * bmf_type list;;

val transform_annotations :
    ('a -> walk_kind -> ('a,'b) transform_bmf_action)
    -> 'a -> ('b -> 'b -> 'b) -> 'b -> annotation list
    -> 'b * annotation list;;

(************************************************************************)
(* Deep copying								*)
(************************************************************************)

val deep_copy_ident : bmf_ident -> bmf_ident;;
val deep_copy_frame : bmf_frame -> bmf_frame;;
val deep_copy_stmt : bmf_stmt -> bmf_stmt;;
val deep_copy_expr : bmf_expr -> bmf_expr;;
val deep_copy_type : bmf_type -> bmf_type;;
val deep_copy_ann : annotation -> annotation;;

