(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
(*****************************************************************
 *			 Generic expressions		  
 *							  
 * Parametrized by the atom type			  
 ******************************************************************)

(*
module type Ordered_type_with_printer = 
  sig 
    type t 
    val compare : t -> t -> int 
    val to_string : t -> string
   end
*)

open Bsct_base

module type E =
  sig
    module ATOM : AtomType
    type atom = ATOM.t

    exception Invalid_s_expr of Sx.t

    type cnst =
      | Int of int
      | Real of float
      | String of string
      | Bool of bool
      | Void

    type op =
	Plus
      | Times
      | Neg
      | Div
      | Sqrt
      | Not
      | And
      | Or
      | Leq
      | Eq
      |	Lt
      |	Gt
      |	Geq
      |	Apply

    type expr = 
	Atom of atom 
      | Const of cnst 
      | Op of op * expr list

    val op_arg : expr -> expr
    val op_arg1 : expr -> expr
    val op_arg2 : expr -> expr
    val op_args : expr -> expr list
    val args_by_op : op -> expr -> expr list
    val make_nary_expr : op -> expr -> expr -> expr list -> expr
    val make_neg : expr -> expr
    val make_not : expr -> expr
    val make_plus : expr list -> expr
    val make_minus : expr -> expr -> expr
    val make_times : expr list -> expr
    val make_and : expr list -> expr
    val make_or : expr list -> expr
    val make_binary : op -> expr -> expr -> expr
    val collect_atoms : expr -> atom list
    val rename : atom list -> atom list -> expr -> expr

    val to_sexpr : expr -> Sx.t
    val to_string : expr -> string

    module Type :
        sig
          type t =
            | Atom
            | Num
            | Int
            | Real
            | String
            | Bool
            | Void
            | Top
	  val type_lub : t -> t -> t
	  val type_lub_list : t list -> t
	  val apply_op : op -> t list -> t
        end
    val expr_type : expr -> Type.t
  end

module Make (A : AtomType) : E with module ATOM = A






