(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl
open Containers
open Bsct_base
open Bmf

(* *****************************************************************
   Reaching definition analysis
   ***************************************************************** *)

(* Data type for representing the definitions. *)

type def_t = | Def_entry of string | Def_write of Bmf.bmf_stmt * Bmf.bmf_expr
val defs_equal : def_t -> def_t -> bool

module Def_element : (ELEMENT with type t = def_t)
module Def_ordinator : (ORDINATOR with type elt = def_t)

(***************************************************************************)

type result_t = Int_set.t Hybrid_dataflow.Result.t

(* Compute reaching defs for the stmt tree rooted at S. The result is
   (ord,in_h,out_h). ord is an enumerator that maps defs onto ints. in_h is
   a hashtable : bmf_stmt -> Int_set.t which are the defs reaching a stmt.
   out_h is hashtable of the defs leaving a stmt. *)

val compute_reaching_defs : 
    bmf_stmt (*S*) ->
      Def_ordinator.t (*ord*) *
	result_t (*in_h*) * result_t (*out_h*)

(* Takes one of the hashtables returned by compute_reaching_defs and an
   alias map and returns a function, F. When F is applied to a stmt and a
   name, it returns all of the definitions that enter/leave (depending
   upon which hashtable is used) the stmt that _may_ write to the name. *)
	
val defs_at_stmt_by_name :
    Def_ordinator.t ->
      result_t -> Hybrid_aliases.aliases_map ->
     	(bmf_stmt -> string -> Int_set.t)

val test_proc : bmf_ident -> unit
