(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bmf
open Bmf_join
open Bb
open Bb_decl

type sched_what_type = Sched_just_dense | Sched_everything;;
(* 0 = search and nested join. 1 = 0 + scatter/gather & two-finger *)
type sched_level_type = int;;

(* Entry points *)
val schedule_procs : sched_what_type -> sched_level_type -> 
  bmf_ident list -> bmf_ident list;;
val schedule_proc : sched_what_type -> sched_level_type -> 
  bmf_ident -> bmf_ident;;

(* Low-level routines for generating join implementations. *)

val split_join_terms : 
    ((*bbm:*) black_box_map) ->
      join_term_type list ->
	((*non_trivial_terms:*) join_term_type list
	  * (*trivial_terms:*) join_term_type list
	  * (*lookup_terms:*) join_term_type list)

val gen_join_nested_loop : 
    join_type ->
      ((*loop_term:*) join_term_type) ->
	((*search_terms:*) join_term_type list) ->
	  ((*body:*) bmf_stmt) ->
	    bmf_stmt

val gen_join_term_loop :
    join_type ->
      ((*loop_term:*) join_term_type) ->
	((*body:*) bmf_stmt) ->
	  bmf_stmt

val gen_join_2_finger :
    ((*term1:*) join_term_type) ->
      ((*term2:*) join_term_type) ->
	((*body:*) bmf_stmt) ->
	  bmf_stmt

type scatter_gather_type = {
    (* Temporary array for scatter *)
    sg_id : bmf_ident;
    (* Initialization code for scatter *)
    sg_init : bmf_stmt;
    (* Scatter code *)
    sg_scatter : bmf_stmt;
    (* Gather code *)
    sg_gather : bmf_stmt;
    (* Tested body of join *)
    sg_body : bmf_stmt
  }	

val gen_join_scatter :
    ((*bbm:*) black_box_map) ->
      join_type ->
      	((*term:*) join_term_type) ->
	  ((*body:*) bmf_stmt) ->
	    scatter_gather_type

val insert_determineds :
    join_type ->
      ((*terms:*) join_term_type list) ->
	((*body:*) bmf_stmt) ->
	  bmf_stmt

val insert_determined :
    join_type ->
      ((*term:*) join_term_type) ->
	((*body:*) bmf_stmt) ->
	  bmf_stmt

(* Make a new join from an old join by updating the list of terms and the
   body. *)
val update_join :
    join_type ->
      ((*new_terms:*) join_term_type list) ->
	((*body:*) bmf_stmt) ->
	  join_type
      
