(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bsct_base

module type L_affine =
  sig
    module L : Lambda_expr.L
    type atom = L.atom
    type expr = L.expr
    type key = L.key

    module A : 
	Affine.A with type atom = atom
    	and type expr = expr

    (*******
      We take a lambda La0...La(n-1) P(a) and
      integers i and j and determine which of the
      following three is true:
      -- the predicate is overconstrained. In this case
	 the return of analyze() is "Overconstrained"
      -- ai = function of aj and some free param for all aj.
         In this case the return of analyze() is
         Universal(lbda) where lbda is of the form:
	    Laj Lai Lt ai = f(aj,t)
	 or, if aj uniquelly determines ai:
         Universal_unique(lbda) with lbda of the form:
            Laj Lai ai = f(aj)
      -- aj is bound. The return is a parametric representation
         of ai and aj in block echelon form given as
         Bound(lbdaj,lbdai) where
	    lbdaj = Laj Lt aj=f(t)
            lbdai = Lai Lt Lu ai=g(t,u)
         There is a special case when u is empty. That is for every
         aj that satisfies the original system of constraints
         there is a unique ai. The result in this case is given by
         Bound_unique(lbdaj,lbdai) where
	    lbdaj = Laj Lt aj=f(t)
            lbdai = Lai Lt ai=g(t)
            
     *******)

    type analysis_result =
	{ param_t : key;
	  param_u : key;
	  key_input : key;
	  key_output : key;
	  summary_lbda : L.t;
	  desc : analysis_result_desc }

    and analysis_result_desc =
      |	Overconstrained
      |	Universal of L.t          (* Lin Lout Lu . out= f(in,u) *)
      |	Universal_unique of L.t   (* Lin Lout . out = f(in) *)
      |	Bound of L.t * L.t        (* Lin Lt . in=f(t) ; Lout Lt Lu . out=g(t,u) *)
      |	Bound_unique of L.t * L.t (* Lin Lt . in=f(t) ; Lout Lt . out=g(t) *)

    (*****************
      (analyze lbda key_out key_in key_t key_u)
      considers key_out variables as a function of key_in
      variables. key_t and key_u name the free params in the
      output

      (analyze1 lbda key_out key_t)
      returns key_to variables as a function of a free param
      key_t
      ****************)      
    val analyze : L.t -> (*out*) key -> (*in*) key -> 
      (*t*) key -> (*u*) key -> analysis_result

    val analyze1 : L.t -> (*out*) key -> (*t*) key -> analysis_result

  end

module Make(Lbda :  Lambda_expr.L) : L_affine with module L = Lbda
	
