#########################################################################
#	Copyright (C) 1998. Bernoulli Project, Cornell University.	#
#########################################################################
use Getopt::Std;

###########################################################################

$opt_o = "-";
$opt_M = 0;
getopts('o:M');

$stdin_name = "<standard input>";
$stdout_name = "<standard output>";

if ( $opt_o eq "-" ) {
    $opt_o = $stdout_name;
}

if ( $opt_M ) {
    ;
} elsif ( $opt_o eq $stdout_name ) {
    $output = *STDOUT;
} else {
    $output = $curr_handle++;
    open($output, ">$opt_o") 
	|| die "Can't open $opt_o";
}

if ( $#ARGV == -1 ) {
    if ($opt_M) {
	die "Can't use -M on stdin."
	}
    push(@ARGV,"-");
}

foreach $input_file ( @ARGV ) {
    if ( $input_file eq '-' ) {
	$input_file = $stdin_name;
    }
    process_file($input_file, $output);
    if ($opt_M) { 
	print (generate_make_rule($input_file, $opt_o)); 
    }
}


###########################################################################

$curr_handle = "fh00";

sub process_file ($$) {
    my($input_file, $output) = @_;
    local $input;
    local $input_line = 0;

    if ( $input_file eq $stdin_name ) {
	$input = *STDIN;
    } else {
	$input = $curr_handle++;
	open($input, "<$input_file") 
	    || die "Can't open $input_file";
    }

    add_included_list($input_file);

    if (!$opt_M) { printf $output "# 1 \"%s\"\n", $input_file; }
    while (<$input>) {
	$input_line++;
	if (/^\#include +"(.*)"/) {
	    process_file(find_file($1), $output);
	    if (!$opt_M) {
		printf $output "# %d \"%s\"\n", $input_line, $input_file;
	    }
	} else {
	    if (!$opt_M) {
		print $output $_;
	    }
	}
    }

    close($input);
}

###########################################################################

sub find_file ($) {
    my ($raw_input_file) = @_;
    return $raw_input_file;
}
    
###########################################################################

%included = ();
$num_included = 0;

sub add_included_list($) {
    my ($input_file) = @_;
    if (! exists($included{$input_file}) ) {
	$included{$input_file} = $num_included++;
    }
}

sub generate_included_list () {
    local @zzz = ();
    foreach $file (keys %included) {
	$zzz[$included{$file}] = $file;
    }
    return @zzz;
}

sub generate_make_rule () {
    my ($input_file, $output_file) = @_;
    local $result = $output_file . " :";
    foreach $file (generate_included_list()) {
	$result = $result . " " . $file;
    }
    $result = $result . "\n";
    return $result;
}

sub ocamlprep_error () {
    my ($source_file, $source_line, $msg) = @_;
    fprintf($stderr, "%s:%d: %s\n", $source_file, $source_line, $msg);
    exit(1);
}
