(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bsct_base

(*********************************************************)
(* Relational expressions                                *)
(*                                                       *)
(* basically, these are SELECT...FROM...WHERE queries    *)
(*                                                       *)
(*********************************************************)

type t
type lambda_expression = Expr_string.L.t
	  
type binding = {
    relation : string;
    index_vars : string list;
    value_var : string;
    flag_var : string }
      
val make : (*iter:*) string list -> (*out:*) string list -> 
      (* bindings *) (string * binding) list -> 
      (* equalities *) lambda_expression ->
      (* inequalities *) lambda_expression ->
      (* predicate *) lambda_expression -> t

val verify : t -> string -> t
val from_sexpr : Sx.t-> t
val to_sexpr : t -> Sx.t

val has_binding_p : t -> string -> bool
val get_binding : t -> string -> binding
val set_binding : t -> string -> binding -> t
val remove_binding : t -> string -> t

val rename_binding : t -> string -> string -> t

val get_binding_names : t -> string list
val get_all_bindings : t -> (string * binding) list
val get_eqs : t -> lambda_expression
val get_ineqs : t -> lambda_expression
val get_pred : t -> lambda_expression

val set_eqs : t -> lambda_expression -> t
val set_ineqs : t -> lambda_expression -> t
val set_pred : t -> lambda_expression -> t

val get_out_vars : t -> string list
val set_out_vars : t -> string list -> t

val get_iter : t -> string list
val set_iter : t -> string list -> t

(* get list of associations between 
   variables used in the bindings and binding names
   *)
type bound_var_type =
    Index of string
  | Value of string
  | Flag of string 
  | Relation of string
    
type var_type = 
    Loop | Bound of bound_var_type

val get_inverse_binding_list : 
    t -> (string * var_type) list

val get_value_vars : t -> string list
val get_index_vars : t -> string list (* note: this returns loop vars
					 as well as array indices *)
val get_flag_vars : t -> string list
val get_relation_names : t -> string list

val get_binding_name4var_type : bound_var_type -> string

(* get a lambda-expression for an array access fn *)
(* the result has the keys: symb, iter and access_name
val get_array_access_fn : t -> (* access name:*) string -> lambda_expression

(* scheme interface *)
val attach : Scheme.env_type-> string -> unit


