/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
/*
 * Communication buffers
 */

#ifndef _commbuffer_h
#define _commbuffer_h

#include "hash.h"

/***********************************************************************
 *                      Comm. buffer structure
 ***********************************************************************/
typedef struct commbuffer_s commbuffer_t;
typedef struct commbuffer_s *commbuffer;

typedef enum { CBUF_INT, CBUF_REAL, CBUF_VOID } cbuf_val_type;

/*
  IMPORTANT: All indices start from 0
  */

struct commbuffer_s {
    int nproc;
    int nind;
    int arity;
    cbuf_val_type t;
    int *iptr;
    int *ind;
    int *vals;
};

#define nproc_cbuf(c)           ((c)->nproc)
#define arity_cbuf(c)           ((c)->arity)
#define nind_cbuf(c)            ((c)->nind)
#define iptr_cbuf(c)            ((c)->iptr)
#define iptr4q_cbuf(c,q)        (((c)->iptr)[q])


#define ind_cbuf(c)             ((c)->ind)
#define ind_tuple_cbuf(c,ii)    (ind_cbuf(c) + (ii) * arity_cbuf(c))
        /* tuples are laid out "row-wise" */

#define ind_tuple_elt_cbuf(c,ii,j) \
                (ind_tuple_cbuf((c),(ii))[j])
#define vals_cbuf(c)            ((c)->vals)

#define typed_vals_cbuf(T,c)    ((T *)vals_cbuf(c))
#define val4ii_cbuf(T,c,ii)     (typed_vals_cbuf(T,c)[ii])
#define int_val4ii_cbuf(c,ii)   (val4ii_cbuf(int,(c),(ii)))
#define real_val4ii_cbuf(c,ii)  (val4ii_cbuf(double,(c),(ii)))
#define void_val4ii_cbuf(c,ii)  (abort())


#define arity_cbuf(c)           ((c)->arity)
#define val_type_cbuf(c)        ((c)->t)

commbuffer make_cbuf (int nproc, int nind, int arity, cbuf_val_type t);
commbuffer make_cbuf_from_iptr (int nproc, int *iptr, int arity, cbuf_val_type t);
           /* Note: this function does NOT reuse the iptr array within the buffer */

commbuffer pack_cbuf (int nproc, int arity, cbuf_val_type t, hash_table h);
void free_cbuf(commbuffer c);


/***********************************************************************
 *                         Utilities
 ***********************************************************************/
void counts_to_iptr (int nproc, int counts[], int iptr[]);
     /* can use the same array for both counts and iptr */

int count_non_empty_procs (commbuffer c);


/***********************************************************************
 *                       Searching the buffer
 ***********************************************************************/

int find_q4ii_cbuf (commbuffer c, int ii);
int find_ii4tuple_cbuf (commbuffer c, int q, int *t); /* -1 if not found */

/***********************************************************************
 *                       Making more buffers
 ***********************************************************************/

commbuffer proj_cbuf (commbuffer c, int new_arity, int keep_vals);
void copy_common_prefix_fields_cbuf (commbuffer cto, commbuffer cfrom);
void copy_values_cbuf (commbuffer cto, commbuffer cfrom);

/***********************************************************************
 *				Misc.
 ***********************************************************************/
void verify_cbuf(int, commbuffer c);

#endif
