/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/
/*
 * Communication routines
 */

#ifndef _rt_comm_h
#define _rt_comm_h

#include "mpi.h"

#include "commbuffer.h"

/**********************************************************************
*                 Manipulating tags
**********************************************************************/

/* A "user_tag" is the argument to send_cbuf and recv_cbuf These can
   be called from top level program or from the exchange routines. The
   TAG_EXCHANGE bit differentiates these two cases.  This way the
   namespace of the exhange() routine is different from the top-level
   user namespace.

   
   Tuple indices and data are sent separately. TAG_IS_IND
   differentiates between these.
   */

typedef enum 
{
    TAG_IS_IND=0x1,
    TAG_EXCHANGE=0x2,
} tag_kind;

#define make_exchange_tag(e)	(((e)<<8) | TAG_EXCHANGE)
#define make_user_tag(u)	((u)<<8)
#define make_user_ind_tag(u)	(((u)<<8) | TAG_IS_IND)
#define make_user_val_tag(u)	(((u)<<8))

#define kind_of_tag(t)		((tag_kind) ((t) & 0xf))
        /* whether IND or VAL */

#define is_exchange_tag(t)	((t) & TAG_EXCHANGE)
#define is_ind_tag(t)		((t) & TAG_IS_IND)
#define user_tag(t)		((t)>>8)

/**********************************************************************
*               Sending/Receiving/Exchanging comm-buffers
**********************************************************************/

commbuffer exchange_cbuf (int me, int nproc, commbuffer c);

/* send is non-blocking, receive is blocking.
   the send buffer can be reused after the receive is 
   done
   */

void send_cbuf (int me, int nproc, commbuffer c, int tag);
void recv_cbuf (int me, int nproc, commbuffer c, int tag);


/**********************************************************************
*                          MPI shortcuts
**********************************************************************/

#define reduce_int_sum(from,to,count)		\
        MPI_Allreduce ((from),(to),(count),	\
		    MPI_INT,MPI_SUM,		\
		    MPI_COMM_WORLD)

#endif



