(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl
open Containers

open Bmf
open Bb
open Bb_decl

open Scheduler_stages
open Scheduler

(************************************************************************
Entry points and query finder.

Code to find queries and invoke [[schedule_query]].
************************************************************************)
let rec schedule_procs (verbose_p: bool) (decls: bmf_ident list) = (
  List.rev
    (List.map (schedule_proc verbose_p)
       (List.rev decls))
: bmf_ident list)

and schedule_proc (verbose_p: bool) (decl: bmf_ident) = (
    match decl with
      Ident_valid(name,typ,Ident_proc(Frame(args),res,body),ann) ->
        let body' =
            walk_outer_stmt (add_array_decls (res::args)
                                String_map.empty)
	        verbose_p body
        in
            Ident_valid(name,typ,Ident_proc(Frame(args),res,body'),ann)
    | _ -> decl
: bmf_ident)

and walk_outer_stmt (bbm: black_box_map) (verbose_p: bool) (s: bmf_stmt) = (
    let s0 = walk_outer_stmt bbm verbose_p
    in
        match s with
          (Stmt_assign(_,_)|Stmt_call(_,_,_)|Stmt_nop) -> s
        | Stmt_seq(first,second) -> Stmt_seq(s0 first, s0 second)
        | Stmt_for(i,lb,ub,step,body) -> Stmt_for(i,lb,ub,step,s0 body)
        | Stmt_while(pred,body) -> Stmt_while(pred,s0 body)
        | Stmt_if(pred,thn,els) -> Stmt_if(pred,s0 thn,s0 els)
        | Stmt_let(Frame(decls),body) ->
            Stmt_let(Frame(decls),
                walk_outer_stmt (add_array_decls decls bbm)
		       verbose_p body)
        | Stmt_do(pred,body) ->
            if Bmf_query.stmt_is_query s then
	      let query = (Bmf_query.stmt2query s)
	      in let hlp = schedule_query bbm query verbose_p
	      in hlp2bmf hlp
            else
                Stmt_do(pred,s0 body)
        | Stmt_directive(ann,s') -> Stmt_directive(ann, s0 s')
: bmf_stmt)

(************************************************************************)
(*			  Command line processing			*)
(************************************************************************)

let _ =
  register_switch "-jsc"
    "-jsc (bmf) -- Join scheduling"
    (fun top_decls args ->
      let name = Bsct_toplevel.get_arg_name (List.hd args)
      and (options,param,args') = Bsct_toplevel.parse_arg false args
      in let (verbose_p,options) = Bsct_toplevel.extract_option options 'v'
      in let _ = Bsct_toplevel.ignore_options options name
      in let top_decls' = schedule_procs verbose_p top_decls
      in (top_decls',args'))
