(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Alpha;;
open Bmf;;
open Scheduler_types;;

(************************************************************************
This section describes the linear portion of the state that is maintained
during planning. The fundamental type used for maintaining this state is
[[linear_state_type]].
************************************************************************)
type linear_state_type = {
    ls_H : matrix;             (* <== The system *)
    ls_x0 : vector;            (* <==            *)
    ls_index_to_field_v: field_name array; (* <==            *)
    ls_index_of_field : field_name -> int;
    ls_i : int; (* The first row in H not already in echelon *)
    ls_j : int; (* The first col in H not already in echelon *)
    ls_initial_i : int; (* The upper corner of H after preprocessing *)
    ls_initial_j : int;
    ls_old_is : int list;
		(* List of previous values of i. Can be used to
		   identify sets of rows in each join *)
    ls_equiv_classes : int list list
		(* Equiv classes within $\tilde{H}$ *)
}
;;

val output_linear_state_type : out_channel -> linear_state_type -> unit;;

(* And this is how the state is advanced, *)
type linear_advance_kinds =
        Linear_initial 
    |   Linear_subseq
;;

val make_initial_linear_state:
    matrix -> vector -> field_name array
	-> linear_state_type
;;

val advance_linear_state:
    linear_advance_kinds -> linear_state_type -> int list
	-> linear_state_type
;;

val expr_of_row:
    string array -> linear_state_type -> int
	-> bmf_expr
;;
