(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bmf
open Bmf_query
open Bmf_join
open Bb_decl

open Scheduler_types
open Scheduler_hier
open Scheduler_linear

type global_state_type

val make_global_state:
    black_box_map -> query -> global_state_type

val global_state_bbm:
    global_state_type -> black_box_map
val global_state_query:
    global_state_type -> query
val global_state_all_fields:
    global_state_type -> bound_type
val global_state_all_rels:
    global_state_type -> string list
val global_state_qb_of_rel:
    global_state_type -> string -> query_binding

(* fixme *)
val global_state_t_var_name_v:
    global_state_type -> string array

val joinable_field_p:
    global_state_type -> field_name -> bool


type range_info_type

val range_info_of_symbolics :
    range_info_type -> bmf_expr
val range_info_of_column:
    range_info_type -> int -> join_bounds_type
val range_info_of_row:
    range_info_type -> int -> join_bounds_type

type step_state_type

val make_step_state:
    global_state_type -> step_state_type

val update_ss_ls:
    linear_state_type -> step_state_type -> step_state_type

val step_state_bound:
    step_state_type -> bound_type
val step_state_ls:
    step_state_type -> linear_state_type
val step_state_ready_hterms:
    step_state_type -> hterm_kind list

val output_step_state : out_channel -> step_state_type -> unit

(* fixme *)
val determined_field_p:
    global_state_type -> step_state_type -> field_name
	-> bool

(************************************************************************)

type hlp_type =
    global_state_type * stage_type

and stage_type =
  | Stage_constant of constant_stage_type * stage_type
  | Stage_invariant of invariant_stage_type * stage_type
  | Stage_join of join_stage_type * stage_type
  | Stage_determined of determined_stage_type * stage_type
  | Stage_unjoinable of unjoinable_stage_type * stage_type
  | Stage_body of body_stage_type

and constant_stage_type 
and invariant_stage_type 
and join_stage_type
and determined_stage_type
and unjoinable_stage_type
and body_stage_type

val hlp2bmf : hlp_type -> bmf_stmt

(************************************************************************)

val make_constant_stage: 
    global_state_type -> step_state_type ->
      (step_state_type * constant_stage_type)

(************************************************************************)

val make_invariant_stage:
    global_state_type -> step_state_type -> int list ->
      (step_state_type * invariant_stage_type)

(************************************************************************)

val make_join_stage:
    global_state_type -> step_state_type -> int list -> hterm_kind list ->
      (step_state_type * join_stage_type)

(************************************************************************)

val make_determined_stage:
    global_state_type -> step_state_type -> hterm_kind ->
      (step_state_type * determined_stage_type)

(************************************************************************)

val make_unjoinable_stage:
    global_state_type -> step_state_type -> hterm_kind ->
      (step_state_type * unjoinable_stage_type)

(************************************************************************)

val make_body_stage:
    global_state_type -> step_state_type ->
      body_stage_type
