(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Printf;;

open Ssisl
open Containers

open Bb;;

(* Turn verbose messages on or off. *)
let verbose = ref false;;

(* field_name *)

type field_name =
        Field_rel of string * string
    |   Field_loop of string
    |   Field_inv of string
;;

let output_field_name outch f =
  match f with
    Field_rel(r,f') -> fprintf outch "%s.%s" r f'
  | Field_loop(i) -> fprintf outch "_.%s" i
  | Field_inv(n) -> fprintf outch "%s" n
;;

module Fn_element =
  struct
    type t = field_name
    let compare = Pervasives.compare
    let equal = (=)
    let hash = Containers.hash
  end

module Fn_set = Set(Fn_element)
module Fn_map = Map(Fn_element)
module Fn_hashtbl = Hashtbl(Fn_element)

(* bound_type *)

type bound_type = BOUND of field_name list;;

let bound_of_list lst = BOUND(lst);;

let list_of_bound (BOUND(lst)) = lst;;

let initial_bound = BOUND([]);;

let add_bound (BOUND(bound)) (BOUND(new_bound)) =
    BOUND(List'.union new_bound bound);;

let all_bounded_p (BOUND(bound)) (BOUND(targ_fields)) =
    (List'.subsetp targ_fields bound);;

let is_bound_p (BOUND(bound)) name =
    List.mem name bound;;

let unbound (BOUND(bound)) lst =
    List'.difference lst bound;;

let output_bound_type outch (BOUND(lst)) =
  fprintf outch "BOUND(%a)" (Io'.output_list output_field_name) lst
    ;;

(* named_access_method *)

type named_access_method = {
    nam_rel : string;
    nam_unnamed : access_method;
    nam_in_fields : field_name list;
    nam_out_field : field_name
};;

let make_named_access_method r am = {
    nam_rel = r;
    nam_unnamed = am;
    nam_in_fields =
	List.map (fun f -> Field_rel(r,f)) am.am_in_fields;
    nam_out_field =
	Field_rel(r,am.am_out_field)
};;

let output_named_access_method outch nam =
  fprintf outch "%s.%s" nam.nam_rel nam.nam_unnamed.am_name
    ;;
