(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Ssisl
open Containers

open Bb;;

val verbose : bool ref;;

(************************************************************************
[[field_name]] is used to name fields within relations. There are three
types of field names,

o those naming array index/field names, 
o those naming loop variable names, and
o those naming query invariant constants.
************************************************************************)    
type field_name =
        Field_rel of string * string
    |   Field_loop of string
    |   Field_inv of string
;;

val output_field_name : out_channel -> field_name -> unit;;

module Fn_element : (COMPLETE_ELEMENT with type t = field_name)
module Fn_set : (SET with type elt = field_name)
module Fn_map : (MAP with type key = field_name)
module Fn_hashtbl : (HASHTBL with type key = field_name)

(************************************************************************
The [[bound_type]] is used for representing the list of bound field names.
************************************************************************)

type bound_type;;

val bound_of_list : field_name list -> bound_type;;
val list_of_bound : bound_type -> field_name list;;
val initial_bound : bound_type;;
val add_bound : bound_type -> bound_type -> bound_type;;
val all_bounded_p : bound_type -> bound_type -> bool;;
val is_bound_p : bound_type -> field_name -> bool;;
val unbound : bound_type -> field_name list -> field_name list;;

val output_bound_type : out_channel -> bound_type -> unit;;

(************************************************************************
"Named" access methods

Relation name/Access method pairs.
************************************************************************)

type named_access_method = {
    nam_rel : string;
    nam_unnamed : access_method;
    nam_in_fields : field_name list;
    nam_out_field : field_name
};;

val make_named_access_method : string -> access_method ->
    named_access_method;;

val output_named_access_method : out_channel -> named_access_method -> unit;;
