;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;	Copyright (C) 1998. Bernoulli Project, Cornell University.     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; (define-macro (name forms ...) expander-body ...)

(%%define-macro define-macro
  (lambda (form)
    (let ((binding (cadr form))
	  (body-forms (cddr form)))
      (let ((name (car binding))
	    (args (cdr binding)))
	(list '%%define-macro
	      name
	      (list 'lambda (list 'forms)
		    (list 'apply
			  (cons 'lambda (cons args body-forms))
			  (list 'cdr 'forms))))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 6.3 Pairs and Lists

;;; Some of the caddd...r and cdddd...r routines are commented because they
;;; are implemented as primitives.

(define (caar x) (car (car x)))
;(define (cadr x) (car (cdr x)))
(define (cdar x) (cdr (car x)))
;(define (cddr x) (cdr (cdr x)))

(define (caaar x) (car (car (car x))))
(define (caadr x) (car (car (cdr x))))
(define (cadar x) (car (cdr (car x))))
;(define (caddr x) (car (cdr (cdr x))))
(define (cdaar x) (cdr (car (car x))))
(define (cdadr x) (cdr (car (cdr x))))
(define (cddar x) (cdr (cdr (car x))))
;(define (cdddr x) (cdr (cdr (cdr x))))

(define (caaaar x) (car (car (car (car x)))))
(define (caaadr x) (car (car (car (cdr x)))))
(define (caadar x) (car (car (cdr (car x)))))
(define (caaddr x) (car (car (cdr (cdr x)))))
(define (cadaar x) (car (cdr (car (car x)))))
(define (cadadr x) (car (cdr (car (cdr x)))))
(define (caddar x) (car (cdr (cdr (car x)))))
;(define (cadddr x) (car (cdr (cdr (cdr x)))))
(define (cdaaar x) (cdr (car (car (car x)))))
(define (cdaadr x) (cdr (car (car (cdr x)))))
(define (cdadar x) (cdr (car (cdr (car x)))))
(define (cdaddr x) (cdr (car (cdr (cdr x)))))
(define (cddaar x) (cdr (cdr (car (car x)))))
(define (cddadr x) (cdr (cdr (car (cdr x)))))
(define (cdddar x) (cdr (cdr (cdr (car x)))))
;(define (cddddr x) (cdr (cdr (cdr (cdr x)))))

(define (caaaaar x) (car (car (car (car (car x))))))
(define (caaaadr x) (car (car (car (car (cdr x))))))
(define (caaadar x) (car (car (car (cdr (car x))))))
(define (caaaddr x) (car (car (car (cdr (cdr x))))))
(define (caadaar x) (car (car (cdr (car (car x))))))
(define (caadadr x) (car (car (cdr (car (cdr x))))))
(define (caaddar x) (car (car (cdr (cdr (car x))))))
(define (caadddr x) (car (car (cdr (cdr (cdr x))))))
(define (cadaaar x) (car (cdr (car (car (car x))))))
(define (cadaadr x) (car (cdr (car (car (cdr x))))))
(define (cadadar x) (car (cdr (car (cdr (car x))))))
(define (cadaddr x) (car (cdr (car (cdr (cdr x))))))
(define (caddaar x) (car (cdr (cdr (car (car x))))))
(define (caddadr x) (car (cdr (cdr (car (cdr x))))))
(define (cadddar x) (car (cdr (cdr (cdr (car x))))))
;(define (caddddr x) (car (cdr (cdr (cdr (cdr x))))))
(define (cdaaaar x) (cdr (car (car (car (car x))))))
(define (cdaaadr x) (cdr (car (car (car (cdr x))))))
(define (cdaadar x) (cdr (car (car (cdr (car x))))))
(define (cdaaddr x) (cdr (car (car (cdr (cdr x))))))
(define (cdadaar x) (cdr (car (cdr (car (car x))))))
(define (cdadadr x) (cdr (car (cdr (car (cdr x))))))
(define (cdaddar x) (cdr (car (cdr (cdr (car x))))))
(define (cdadddr x) (cdr (car (cdr (cdr (cdr x))))))
(define (cddaaar x) (cdr (cdr (car (car (car x))))))
(define (cddaadr x) (cdr (cdr (car (car (cdr x))))))
(define (cddadar x) (cdr (cdr (car (cdr (car x))))))
(define (cddaddr x) (cdr (cdr (car (cdr (cdr x))))))
(define (cdddaar x) (cdr (cdr (cdr (car (car x))))))
(define (cdddadr x) (cdr (cdr (cdr (car (cdr x))))))
(define (cddddar x) (cdr (cdr (cdr (cdr (car x))))))
;(define (cdddddr x) (cdr (cdr (cdr (cdr (cdr x))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 6.5 Numbers

(define (zero? x) (= 0 x))
(define (positive? x) (< 0 x))
(define (negative? x) (> 0 x))

(define (odd? x) (= 1 (modulo x 2)))
(define (even?? x) (= 0 (modulo x 2)))

(define (max x . rest)
  (if (null? rest) 
      x
      (let* ((y (car rest))
	     (z (if (> x y) x y))
	     (zz (if (or (inexact? x) (inexact? y))
		     (exact->inexact z)
		     z)))
	(apply max zz (cdr rest)))))

(define (min x . rest)
  (if (null? rest) 
      x
      (let* ((y (car rest))
	     (z (if (< x y) x y)))
	     (zz (if (or (inexact? x) (inexact? y))
		     (exact->inexact z)
		     z))
	(apply min zz (cdr rest)))))

;;; todo:
;;; gcd lcm
;;; numerator denominator
;;; rationalize

(define (expt z1 z2) (exp (* z2 (log z1))))

;;; todo:
;;; make-rectangle
;;; make-polar
;;; real-part
;;; imag-part
;;; magnitude
;;; angle

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 6.6 Characters

(define (char-ci=? a b) (char=? (char-downcase a) (char-downcase b)))
(define (char-ci<? a b) (char<? (char-downcase a) (char-downcase b)))
(define (char-ci>? a b) (char>? (char-downcase a) (char-downcase b)))
(define (char-ci<=? a b) (char<=? (char-downcase a) (char-downcase b)))
(define (char-ci>=? a b) (char>=? (char-downcase a) (char-downcase b)))

(define (char-alphabetic? c) 
  (or (char-upper-case? c) (char-lower-case? c)))
(define (char-numeric? c)
  (memq c '(#\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9)))
(define (char-whitespace? c)
  (memq c '(#\space #\newline #\tab)))
(define (char-upper-case? c) (not (char=? c (char-downcase c))))
(define (char-lower-case? c) (not (char=? c (char-upcase c))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 6.7 Strings

(define (string . rest)
  (let ((str (make-string (length rest))))
    (let lp ((i 0) (rest rest))
      (if (not (null? rest))
	  (begin
	    (string-set! str i (car rest))
	    (lp (+ i 1) (cdr rest)))))
    str))

(define (string-ci=? a b) (string=? (string-downcase a) (string-downcase b)))
(define (string-ci<? a b) (string<? (string-downcase a) (string-downcase b)))
(define (string-ci>? a b) (string>? (string-downcase a) (string-downcase b)))
(define (string-ci<=? a b) (string<=? (string-downcase a) (string-downcase b)))
(define (string-ci>=? a b) (string>=? (string-downcase a) (string-downcase b)))

(define (string->list str)
  (let lp ((i 0) (results '()))
    (if (= i (string-length str))
	(reverse results)
	(lp (+ i 1) (cons (string-ref str i) results)))))

(define (list->string chars) (apply string chars))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 6.8 Vectors

(define (vector . rest)
  (let ((str (make-vector (length rest))))
    (let lp ((i 0) (rest rest))
      (if (not (null? rest))
	  (begin
	    (vector-set! str i (car rest))
	    (lp (+ i 1) (cdr rest)))))
    str))

(define (vector->list v)
  (let lp ((i 0) (results '()))
    (if (= i (vector-length v))
	(reverse results)
	(lp (+ i 1) (cons (vector-ref v i) results)))))

(define (list->vector lst) (apply vector lst))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 6.10 Input and output

;;; todo: are these correct in the presence of error and call/cc?

(define (call-with-input-file name proc)
  (let* ((port (open-input-port name))
	 (result (proc result)))
    (close-input-port port)
    result))

(define (call-with-output-file name proc)
  (let* ((port (open-output-port name))
	 (result (proc result)))
    (close-output-port port)
    result))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; quasiquote

(define-macro (quasiquote x) (%quasiquote-0 x))

(define (%quasiquote-? x n)
  (if (= n 0) 
      (%quasiquote-0 x)
      (%quasiquote-n x n)))

(define (%quasiquote-0 x)
  (cond
   ((and (pair? x) (eq? (car x) 'unquote))
    (cadr x))
   ((and (pair? x) (eq? (car x) 'unquote-splicing))
    (error "Unexpected ,@"))
   ((and (pair? x) (eq? (car x) 'quasiquote))
    (list 'list (list 'quote 'quasiquote)
	  (%quasiquote-n (cadr x) 1)))
   ((pair? x)
    (cons 'append (%qq-list-0 x)))
   ((vector? x)
    (list 'apply 'vector (%quasiquote-0 (vector->list x))))
   (else (list 'quote x))
   ))

(define (%qq-list-0 x)
  (cond
   ((and (pair? x) (eq? (car x) 'unquote))
    (list (cadr x)))
   ((and (pair? x) (eq? (car x) 'unquote-splicing))
    (error "Unexpected ,@"))
   ((pair? x)
    (cons (%qq-bracket-0 (car x))
	  (%qq-list-0 (cdr x))))
   (else
    (list (list 'quote x)))))

(define (%qq-bracket-0 x)
  (cond
   ((and (pair? x) (eq? (car x) 'unquote))
    (list 'list (cadr x)))
   ((and (pair? x) (eq? (car x) 'unquote-splicing))
    (cadr x))
   (else (list 'list (%quasiquote-0 x)))))

(define (%quasiquote-n x n)
  (cond
   ((and (pair? x) (eq? (car x) 'unquote))
    (list 'list (list 'quote 'unquote) (%quasiquote-? (cadr x) (- n 1))))
   ((and (pair? x) (eq? (car x) 'unquote-splicing))
    (error "Unexpected ,@"))
   ((and (pair? x) (eq? (car x) 'quasiquote))
    (list 'list (list 'quote 'quasiquote) (%quasiquote-? (cadr x) (+ n 1))))

   ((pair? x) (cons 'list* (%qq-list-n x n)))

   ((vector? x)
    (list->vector
     (map (lambda (y) (%quasiquote-n y n))
	  (vector->list x))))

   (else (list 'quote x))
   ))

(define (%qq-list-n x n)
  (cond
   ((and (pair? x) (eq? (car x) 'unquote))
    (list (list 'list 'unquote (%quasiquote-? x (- n 1)))))
   ((and (pair? x) (eq? (car x) 'unquote-splicing))
    (error "Unexpected ,@"))
   ((pair? x)
    (cons (%quasiquote-n (car x) n) (%qq-list-n (cdr x) n)))
   (else (list (%quasiquote-n x n)))))

(define (list* x . rest)
  (if (null? rest)
      x
      (cons x (apply list* rest))))


;;; Local Variables: ***
;;; eval: (put '%%define-macro 'scheme-indent-function 1) ***
;;; eval: (put 'define-macro 'scheme-indent-function 1) ***
;;; end: ***
