(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
(* Extremely low-level access to environents, namely as a list of
   frames. These function may disappear if the implementation of
   environment changes sufficiently. *)

(* Returns true if env is empty. IE, no more frames. *)
val env_empty_p : env_type -> bool
(* Returns the "parent" env of an env. This is really CDR for env's. *)
val env_parent : env_type -> env_type
(* Iterates over the bindings of the first frame of an environment. *)
val env_frame_iter : env_type -> (t -> binding_type -> unit) -> unit

(* Low-level routines for directly manipulating environments. These are
   only needed when implementing new special forms. *)
val env_define : env_type -> t -> binding_type -> unit
val env_lookup_once : env_type -> t -> binding_type
val env_lookup : env_type -> t -> binding_type
val env_global_p : env_type -> bool

(* Low-level routines for constructing new environments. *)
val make_global_env : unit -> env_type
val make_closure_env : env_type -> env_type

(* Extract a variable value from an environment. 
   Throws Not_found or Not_a_var if there is a problem. *)
val lookup_var : env_type -> string -> t ref

(* User-level routines for inserting bindings into environments. *)
val define_var : env_type -> string -> t -> unit
val define_special : env_type -> string -> (t -> env_type -> t) -> unit
val define_macro : env_type -> string -> (t -> t) -> unit

(* Routines for inserting primitive function bindings into environments. *)
val define_prim0 : env_type -> string -> (unit -> t) -> unit
val define_prim1 : env_type -> string -> (t -> t) -> unit
val define_prim2 : env_type -> string -> (t -> t -> t) -> unit
val define_prim3 : env_type -> string -> (t -> t -> t -> t) -> unit
val define_prim4 : env_type -> string -> (t -> t -> t -> t -> t) -> unit
val define_prim5 : env_type -> string -> (t -> t -> t -> t -> t -> t) -> unit
val define_primn : env_type -> string -> (t -> t) -> unit

(* User-level routines for obtaining environments. *)

(* Produces an environemnt with no bindings. *)
val null_environment : unit -> env_type

(* Produces an environment with the standard Scheme constructs and
   functions defined. *)
val scheme_environment : unit -> env_type

(* Returns the "default" top-level environment. This is the environment in
   which Scheme expressions are evaluated by default. In particular, this
   is the environment used by the repl and load routines. *)
val interaction_environment : unit -> env_type

(* Routine to change the "default" top-level environment for the duration
   of the execution of thunk. *)
val with_top_level_env : env_type -> (*thunk*)(unit -> t) -> t
