/************************************************************************
 *	Copyright (C) 1998. Bernoulli Project, Cornell University.	*
 ************************************************************************/

%{
%}

%token BO_LIST BO_VECT EO_SEQ
%token QUOTE QUASIQUOTE UNQUOTE UNQUOTE_SPLICING
%token DOT
%token<bool> BOOL
%token<char> CHAR
%token<int> INT
%token<float> FLOAT
%token<string> STRING SYMBOL
%token EOF

%token INF_START
%token INF_END
%token AND OR
%token NOT
%token LT GT LE GE EQ
%token PLUS TIMES DIV MINUS
%token OPENP CLOSEP OPENB CLOSEB
%token COMMA 

%start top_datum
%type<parser_result> top_datum
%type<t> datum

%left OR 
%left AND
%nonassoc NOT
%nonassoc LT GT LE GE EQ
%left PLUS MINUS
%left TIMES DIV
%nonassoc UMINUS

%%

top_datum : datum { Valid($1) } | EOF { E_o_f }
datum : simple_datum {$1} | compound_datum {$1}

simple_datum :
  BOOL { tag_bool $1 }
| INT { tag_int $1 }
| FLOAT { tag_float $1 }
| CHAR { tag_char $1 }
| STRING { tag_string $1 }
| SYMBOL { intern $1 }

compound_datum : list_datum {$1} | vector_datum {$1}

list_datum :
  BO_LIST list_datum_pair EO_SEQ {$2}
| abbrev_prefix datum { (cons $1 (cons $2 nil)) }
| INF_START infix_expr INF_END { $2 }

list_datum_pair :
  /* empty */ { nil }
| datum list_datum_pair { (cons $1 $2) }
| datum DOT datum { (cons $1 $3) }

abbrev_prefix :
  QUOTE { intern "quote" }
| QUASIQUOTE { intern "quasiquote" }
| UNQUOTE { intern "unquote" }
| UNQUOTE_SPLICING { intern "unquote-splicing" }

vector_datum :
  BO_VECT vector_datum_pair EO_SEQ
    { tag_vector (Array.of_list $2) }

vector_datum_pair :
  /* empty */ { [] }
| datum vector_datum_pair { ($1)::($2) }

infix_expr :
  infix_expr AND infix_expr
    { (from_list [(intern "and"); $1; $3]) }
| infix_expr OR infix_expr
    { (from_list [(intern "or"); $1; $3]) }
| infix_expr LT infix_expr
    { (from_list [(intern "<"); $1; $3]) }
| infix_expr GT infix_expr
    { (from_list [(intern ">"); $1; $3]) }
| infix_expr LE infix_expr
    { (from_list [(intern "<="); $1; $3]) }
| infix_expr GE infix_expr
    { (from_list [(intern ">="); $1; $3]) }
| infix_expr EQ infix_expr
    { (from_list [(intern "="); $1; $3]) }
| infix_expr PLUS infix_expr
    { (from_list [(intern "+"); $1; $3]) }
| infix_expr MINUS infix_expr
    { (from_list [(intern "-"); $1; $3]) }
| infix_expr TIMES infix_expr
    { (from_list [(intern "*"); $1; $3]) }
| infix_expr DIV infix_expr
    { (from_list [(intern "/"); $1; $3]) }
| MINUS infix_expr %prec UMINUS
    { (from_list [(intern "-"); $2]) }
| NOT infix_expr %prec NOT
    { (from_list [(intern "not"); $2]) }
| OPENP infix_expr CLOSEP { $2 }
| SYMBOL OPENP infix_expr_list CLOSEP 
    { let f = intern $1 in
        (from_list (f::(to_list $3))) 
    }
| SYMBOL OPENB infix_expr_list CLOSEB 
    { let asym = intern "array_access" 
      and array = intern $1 in
        (from_list [asym ; array ; $3])
    } 
| BOOL { tag_bool $1 }
| INT { tag_int $1 }
| FLOAT { tag_float $1 }
| CHAR { tag_char $1 }
| STRING { tag_string $1 }
| SYMBOL { intern $1 }

infix_expr_list :
  /* empty */ { nil }
| infix_expr { (cons $1 nil) }
| infix_expr COMMA infix_expr_list { (cons $1 $3) }

