(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
let print_scheme_error thunk =
  try
    thunk ()
  with exn ->
    (match exn with
      Arguments_mismatch(tag) -> 
	write_string "Arguments mismatch: ";
	write_string tag
    | Failure(tag) ->
	write_string "Failure: ";
	write_string tag
    | Invalid_form(expr') ->
      	write_string "Invalid form:\n";
      	pretty_print expr'
    | Unbound(sym_name) ->
      	write_string "Unbound symbol: ";
      	write_string sym_name
    | Invalid_argument(name) ->
      	write_string "Invalid argument: ";
      	write_string name
    | Bad_arg(name,x) ->
      	write_string "Invalid argument (";
      	write_string name;
      	write_string "): ";
      	write x
    | Scheme_error(err) ->
      	write_string "Error: ";
      	write_string err
    | Syntax_error -> write_string "Syntax error."
    | Lexical_error -> write_string "Lexical error."
    | exn -> raise exn);
    newline ();
    failwith "print_scheme_error"

let rec repl prompt =

  if prompt <> "" then
    begin
      write_string prompt;
      flush ()
    end;


  let (expr,err_p) = 
    try
      (print_scheme_error read, false)
    with 
      Failure("print_scheme_error") -> (unspecified, true)
    | End_of_file -> (eof, false)
  in

  if eq_p expr eof then
    newline ()
  else if err_p then
    repl prompt
  else
    begin
      let (valu,err_p) = 
	try
	  (print_scheme_error 
	    (fun () -> eval expr (interaction_environment ())), 
	   false)
      	with 
	  Failure("print_scheme_error") ->
	    (unspecified, true)
      in
      
      if not err_p & not (eq_p valu  unspecified) then
      	begin
      	  write valu;
    	  newline ()
      	end;
      flush ();
      repl prompt
    end

let main () =
  (* Parse the command line *)
  let init_file_p = ref true
  and source_files = ref []
  in let anonfun s = source_files := !(source_files)@[s]
  in let _ = 
    Arg.parse 
      [ ("-q",Arg.Unit(fun () -> init_file_p := false),
     	 "Don't load sim.scm.");
	("--",Arg.String anonfun,"Source file.")
      ] anonfun "errmsg"
      
  (* Load the init file. *)
  in let _ =
    if !init_file_p then
      begin
	let init_file_name =
	  Filename.concat
	    (Filename.dirname Sys.argv.(0))
	    "sim.scm"
    	in let err_p = 
	  try
      	    print_scheme_error 
	      (fun () -> load init_file_name false);
	    false
	  with Failure("print_scheme_error") ->
	    true
    	in
    	
    	if err_p then
	  begin
    	    prerr_string "Error loading init file. Can't continue.\n";
    	    exit 1
	  end
      end;

  in
  if !source_files = [] then
    (* Run the interpreter. *)
    Printexc.catch (fun () -> repl "==> ") ()
  else
    (* Load the source files, if any *)
    List.iter (fun s -> load s false) (!source_files)
