(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
module S_expr = Sim.Make(Sim.Case_insensitive)

open S_expr

(* ML functions for computing factorial. *)
let int_fact n =
  let rec iter n result =
    if n = 1 then
      result
    else
      iter (n - 1) (result * n)
  in
  iter n 1

let float_fact n =
    let rec iter n result =
      if n = 1.0 then
	result
      else
	iter (n -. 1.0) (result *. n)
    in
    iter n 1.0

(* A wrapper function that
   o untags the Scheme value, x
   o passes it to the appropriate factorial routine.
   o tags the result and returns it to th Scheme system. *)
let prim_fact x =
  if int_p x then
    tag_int (int_fact (untag_int x))
  else if float_p x then
    tag_float (float_fact (untag_float x))
  else
    raise (Bad_arg("fact",x))

(* define the primitive within the Scheme interaction environment. *)
let _ = define_prim1 (interaction_environment ()) "fact" prim_fact

let _ = main ()
