(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
(* S-expr type *)
type t =
  (* Self-evaluation. E.g., constants *)
    Bool of bool
  | Char of char
  | Float of float
  | Int of int
  | String of string
  (* Lists, Symbols, Vectors *)
  | Nil
  | Pair of pair
  | Vector of t array
  | Symbol of symbol
  (* Unprintable values *)
  | Procedure of procedure
  | Input_port of input_port
  | Output_port of output_port
  | Eof
  | Unspecified

and symbol = string
and pair = { mutable hd : t; mutable tl : t }

(* Procedures *)

and procedure =
    Primitive of primitive_type
  | Closure of closure_type

and primitive_type = {
    prim_name : string;
    prim_func : prim_func_type
  } 

and prim_func_type =
    Prim0 of (unit -> t)
  | Prim1 of (t -> t)
  | Prim2 of (t -> t -> t)
  | Prim3 of (t -> t -> t -> t)
  | Prim4 of (t -> t -> t -> t -> t)
  | Prim5 of (t -> t -> t -> t -> t -> t)
  | Primn of (t -> t)

and closure_type = {
    closure_formals : t;
    closure_env : env_type;
    closure_body : t
  } 

(* Environments *)

and env_type =
    NoEnv
  | Frame of non_empty_env
and non_empty_env = 
    { fr_bindings : bindings; 
      fr_next : env_type; 
      fr_global_p : bool }
and bindings =
    BigBindings of (t,binding_type) Hashtbl.t
  | SmallBindings of (t * binding_type) list ref
and binding_type =
    Special of (t -> env_type -> t)
  | Macro of (t -> t)
  | Variable of (t ref)

and input_port = 
    InPortChannel of (in_channel * Lexing.lexbuf)
  | InPortString of (string * Lexing.lexbuf)
and output_port = 
    OutPortChannel of out_channel
  | OutPortString of string list ref

