(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
(* find_result and find_value :
   This type is used as an alternative to throwing the
   Not_found exception. *)

type 'b find_result =
    Found of 'b
  | Unfound;;

val find_value : (unit -> 'b) -> 'b find_result

(* identity: the identity function. *)

val identity : 'a -> 'a;;

(* complement: negate a predicate *)

val complement1: ('a -> bool) -> 'a -> bool
val complement2: ('a -> 'b -> bool) -> 'a -> 'b -> bool

(* assert: will throw exn if the bool is not true. *)

val zassert : bool -> exn -> unit;;

(* Routine that "look" like Scheme's named let. *)
(*
  recurs 10
      (fun recurs n ->
	if n = 1 then
	  1
	else
	  n * (recurs n-1))
  *)
val recurs : 'a -> (('a -> 'b) -> 'a -> 'b) -> 'b;;


(* Dynamic-wind and friends. *)

val dynamic_wind : (unit -> 'a) -> (unit -> 'b) -> (unit -> 'c) -> 'b
val call_with_dynamic_binding : 'a ref -> 'a -> (unit -> 'b) -> 'b

(* Routines for comparing various type. *)

val compare_ints : int -> int -> int;;
val compare_pair : ('a -> 'b -> int) -> ('c -> 'd -> int) -> 
                   'a * 'c -> 'b * 'd -> int;;

val compare_hash : 'a -> 'a -> int;;

module type OrderedType =
  sig
    type t
    val compare: t -> t -> int
  end

module Tracing :
    sig
      val top : bool (*print tracing?*) -> (unit -> 'a) (*thunk*) -> 'a
      val wrap : (unit -> 'a) (*thunk*) -> string (*name*) ->
	(out_channel -> unit) (*entry printer*) -> 
	  (out_channel -> 'a -> unit) (*exit printer*) -> 'a
      val pass : (unit -> 'a) (*thunk*) ->
	(exn -> bool) (*predicate, which exns to ignore.*) -> 'a
    end

