(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
module type OrderedType =
  sig
    type t
    val compare: t -> t -> int
  end

module Make = 
  functor (OT : OrderedType) ->
    struct
      module S = Map.Make(OT)

      type key = S.key
      type 'a map = 'a S.t

      let compare = OT.compare

      let list_of_bindings m =
  	let l = ref []
  	in let f = fun a b -> (l := (a,b) :: (!l))
  	in ((S.iter f m) ; !l)

      let list_of_range m =
  	List'.long_remove_duplicates
	  (fun x y -> compare x y = 0) identity
	  (List.map (fun (a,b) -> b) (list_of_bindings m))

      let replace x y m =
	(S.add x y (S.remove x m))

      let absorb_list m l =
	let f m (x,y) = S.add x y m in
	List.fold_left f m l

      let absorb m1 m2 =
	absorb_list m1 (list_of_bindings m2)
	
      let it_map f init m =
	let x = ref init in
	S.iter (fun k v -> x := f !x k v) m;
	!x

      let map_it f m init =
	it_map f init m

      let map_size m =
	it_map (fun x _ _ -> x + 1) 0 m

      let remove_list m lst = 
	(List.fold_left
	   (fun m' k -> S.remove k m')
	   m lst)

    end

