(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
module type OrderedType = 
  sig 
    type t 
    val compare : t -> t -> int 
  end

module Make :
  functor(OT : OrderedType) ->
    sig
      module S :
        sig
          type key = OT.t
          type 'a t = 'a Map.Make(OT).t
          val empty : 'a t
          val add : key -> 'a -> 'a t -> 'a t
          val find : key -> 'a t -> 'a
          val remove : key -> 'a t -> 'a t
          val iter : (key -> 'a -> unit) -> 'a t -> unit
          val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
        end
      type key = S.key
      type 'a map = 'a S.t
      val compare : OT.t -> OT.t -> int
      val list_of_bindings : 'a S.t -> (S.key * 'a) list
      val list_of_range : OT.t S.t -> OT.t list
      val replace : S.key -> 'a -> 'a S.t -> 'a S.t
      val absorb_list : 'a S.t -> (S.key * 'a) list -> 'a S.t
      val absorb : 'a S.t -> 'a S.t -> 'a S.t
      val it_map : ('a -> S.key -> 'b -> 'a) -> 'a -> 'b S.t -> 'a
      val map_it : ('a -> S.key -> 'b -> 'a) -> 'b S.t -> 'a -> 'a
      val map_size : 'a S.t -> int
      val remove_list : 'a S.t -> key list -> 'a S.t
    end
