(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
let memoize (keys_equal_p: 'a -> 'a -> bool) (f: 'a -> 'b) =
    let h = Hashtbl.create 101
    in let f' (arg: 'a) = (
	let entries =
	    try
		Hashtbl.find h arg
	    with Not_found ->
		[]
	in let rec iter lst =
	    match lst with
	      (k,v)::_ when keys_equal_p arg k -> v
	    | _::lst' -> iter lst'
	    | [] ->
		let v = f arg
		in
		    Hashtbl.remove h arg;
		    Hashtbl.add h arg ((arg,v)::entries);
		    v
	in
	    iter entries
    : 'b)
    in
	f'
;;

(* flagi = false means use `=' to compare argi.
   flagi = true means use `==' to compare argi. *)

let memoize1 flag f =
    let keys_equal_p =
	match flag with
	  false -> (fun x y -> x = y)
	| true -> (fun x y -> x == y)
    in
	memoize keys_equal_p f
;;

let memoize2 (flag1,flag2) f =
    let keys_equal_p =
	match (flag1,flag2) with
	  false,false -> (fun x y -> x = y)
	| false,true -> (fun (x1,x2) (y1,y2) -> x1 = y1 && x2 == y2)
	| true,false -> (fun (x1,x2) (y1,y2) -> x1 == y1 && x2 = y2)
	| true,true -> (fun (x1,x2) (y1,y2) -> x1 == y1 && x2 == y2)
    in let f' = memoize keys_equal_p (fun (x1,x2) -> f x1 x2)
    in
	(fun x1 x2 -> f' (x1,x2))
;;

let memoize3 (flag1,flag2,flag3) f =
    let keys_equal_p =
	match (flag1,flag2,flag3) with
	  false,false,false -> (fun x y -> x = y)
	| false,false,true ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 = y1 && x2 = y2 && x3 == y3)
	| false,true,false ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 = y1 && x2 == y2 && x3 = y3)
	| false,true,true ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 = y1 && x2 == y2 && x3 == y3)
	| true,false,false ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 == y1 && x2 = y2 && x3 = y3)
	| true,false,true ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 == y1 && x2 = y2 && x3 == y3)
	| true,true,false ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 == y1 && x2 == y2 && x3 = y3)
	| true,true,true ->
	    (fun (x1,x2,x3) (y1,y2,y3) -> x1 == y1 && x2 == y2 && x3 == y3)
    in let f' = memoize keys_equal_p (fun (x1,x2,x3) -> f x1 x2 x3)
    in
	(fun x1 x2 x3 -> f' (x1,x2,x3))
;;

(*
(* Tests *)
let foo x y =
    let v = x ^ (string_of_int y)
    in
	printf__fprintf stdout "<%s>^<%d> -> %s\n" x y v;
	v
;;

let foo' = memoize2 (false,false) foo;;
let foo'' = memoize2 (true,true) foo;;

(* Output for tests
#foo "a" 1;;
<a>^<1> -> a1
- : string = "a1"
#foo' "a" 1;;
- : string = "a1"
#foo' "a" 1;;
- : string = "a1"
#let v = "a";;
v : string = "a"
#foo'' v 1;;
<a>^<1> -> a1
- : string = "a1"
#foo'' v 1;;
- : string = "a1"
#foo'' "a" 1;;
<a>^<1> -> a1
- : string = "a1"
#
*)
*)
