(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
(* a small finite state machine for removing a previous gensym suffix *)
let gensym_base str =
  let rec next_state st i =
    let c =
      if i < 0 then
	'\000'
      else
	(str.[i])
    in
    match st with
      0 when is_digit c -> next_state 1 (i-1)
    | 0 -> next_state 4 i
    | 1 when is_digit c -> next_state 1 (i-1)
    | 1 when c = '_' -> next_state 2 (i-1)
    | 1 -> next_state 4 i
    | 2 when c = '_' -> next_state 3 i
    | 2 -> next_state 4 i
    | 3 -> String.sub str 0 i
    | 4 -> str
    | _ -> failwith "gensym:: broken f.s.m."
  and is_digit c =
    let i = Char.code c
    in
    (Char.code '0') <= i && i <= (Char.code '9')
  in
  next_state 0 (String.length(str)-1)

let gensym_num = ref 0;;
let gensym str =
  let n = !gensym_num in
  incr gensym_num;
  (gensym_base str) ^ "__" ^ (string_of_int n)
		  ;;
    
let gensyms str n =
  Range.fold_right
    (fun _ result -> (gensym str)::result)
    1 n []
    ;;
    
    
(************************************************************************)
(* Routines on vectors							*)
(************************************************************************)
    
(* Functional method for constructing vectors. Only works for len>0. *)
    
let build_vector f len =
  if len <= 0 then
    Array.of_list []
  else
    let v0 = f 0
    in let v = Array.create len v0
    in
    for i = 1 to len-1 do
      v.(i) <- f i
    done;
    v

      
