(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)

module type S =
  sig
    type key
    type stuff
    type t

    val empty : t
    val add : t -> key -> stuff -> t
    val absorb_list : t -> (key * stuff) list -> t
    val find : t -> key -> stuff
    val remove : t -> key -> t
    val fold: (key -> stuff -> 'b -> 'b) -> t -> 'b -> 'b
  end

module type Table_entries =
  sig
    type t 
    type u
    val compare : t -> t -> int
  end

module Make(TE : Table_entries) =
  struct
    module Smap_helper = Map'.Make(TE)
    module Smap = Smap_helper.S

    type key = TE.t
    type stuff = TE.u
    type t = stuff Smap.t

    let empty = Smap.empty
    let add tab k s = Smap.add k s tab
    let absorb_list tab l = Smap_helper.absorb_list tab l
    let find tab k = Smap.find k tab
    let remove tab k = Smap.remove k tab
    let fold f tab a = Smap.fold f tab a
  end
