(************************************************************************)
(*	Copyright (C) 1998. Bernoulli Project, Cornell University.	*)
(************************************************************************)
open Bsct_base

module type S =
  sig

    type atom

    module ATOM : AtomType with type t = atom

    type t =
    	Conjunct of t * t
      | Disjunct of t * t
      | Implies  of t * t
      | Equality of t * t
      | Negation of t
      | Atom of atom
      | Constant of bool

    val compare: t -> t -> int
    exception Invalid_s_expr of Sx.t
    val from_sexpr: Sx.t -> t
    val to_sexpr: t -> Sx.t

    (* Return a list of the conjuncts or disjuncts that appear at the top
       level of the expression e *)
    val to_conjuncts: (*e*) t -> t list
    val to_disjuncts: (*e*) t -> t list

    (* Make a wff from a list of conjuncts or disjuncts *)
    val from_conjuncts: t list -> t
    val from_disjuncts: t list -> t

    (* Return true iff e is a tautology *)
    val tautology_p: (*e*) t -> bool

    (* gist w1 w2 -> w1', where w2 && w1' => w1. Hopefully, w1' is
       "minimal." *)
    val gist: (*w1*) t -> (*w2*) t -> (*w1'*) t

    val simplify: t -> t

    (* Substitute some vars in w. Does _not_ simplify. *)
    val subst: (*bindings*)(atom * t) list -> (*w*)t -> t
  end

module Make(A: AtomType): S with type atom = A.t
module Simple : S with type atom = SimpleAtom.t


